/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "NetworkBroadcastChannelRegistry.h"

#include "ArgumentCoders.h"
#include "Decoder.h"
#include "HandleMessage.h"
#include "NetworkBroadcastChannelRegistryMessages.h"
#include "WebCoreArgumentCoders.h"
#include <WebCore/ClientOrigin.h>
#include <WebCore/MessageWithMessagePorts.h>
#include <wtf/text/WTFString.h>

namespace Messages {

namespace NetworkBroadcastChannelRegistry {

void PostMessage::callReply(IPC::Decoder& decoder, CompletionHandler<void()>&& completionHandler)
{
    completionHandler();
}

void PostMessage::cancelReply(CompletionHandler<void()>&& completionHandler)
{
    completionHandler();
}

void PostMessage::send(UniqueRef<IPC::Encoder>&& encoder, IPC::Connection& connection)
{
    connection.sendSyncReply(WTFMove(encoder));
}

} // namespace NetworkBroadcastChannelRegistry

} // namespace Messages

namespace WebKit {

void NetworkBroadcastChannelRegistry::didReceiveMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
    if (decoder.messageName() == Messages::NetworkBroadcastChannelRegistry::RegisterChannel::name())
        return IPC::handleMessageWantsConnection<Messages::NetworkBroadcastChannelRegistry::RegisterChannel>(connection, decoder, this, &NetworkBroadcastChannelRegistry::registerChannel);
    if (decoder.messageName() == Messages::NetworkBroadcastChannelRegistry::UnregisterChannel::name())
        return IPC::handleMessageWantsConnection<Messages::NetworkBroadcastChannelRegistry::UnregisterChannel>(connection, decoder, this, &NetworkBroadcastChannelRegistry::unregisterChannel);
    if (decoder.messageName() == Messages::NetworkBroadcastChannelRegistry::PostMessage::name())
        return IPC::handleMessageAsyncWantsConnection<Messages::NetworkBroadcastChannelRegistry::PostMessage>(connection, decoder, this, &NetworkBroadcastChannelRegistry::postMessage);
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled message %s to %" PRIu64, IPC::description(decoder.messageName()), decoder.destinationID());
}

} // namespace WebKit
