/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "ViewGestureController.h"

#include "Decoder.h"
#include "HandleMessage.h"
#include "ViewGestureControllerMessages.h"
#if !PLATFORM(IOS_FAMILY) || PLATFORM(MAC)
#include "WebCoreArgumentCoders.h"
#endif
#if PLATFORM(MAC)
#include <WebCore/FloatPoint.h>
#endif
#if !PLATFORM(IOS_FAMILY) || PLATFORM(MAC)
#include <WebCore/FloatRect.h>
#endif

namespace WebKit {

void ViewGestureController::didReceiveMessage(IPC::Connection& connection, IPC::Decoder& decoder)
{
#if PLATFORM(MAC)
    if (decoder.messageName() == Messages::ViewGestureController::DidCollectGeometryForSmartMagnificationGesture::name())
        return IPC::handleMessage<Messages::ViewGestureController::DidCollectGeometryForSmartMagnificationGesture>(connection, decoder, this, &ViewGestureController::didCollectGeometryForSmartMagnificationGesture);
#endif
#if !PLATFORM(IOS_FAMILY)
    if (decoder.messageName() == Messages::ViewGestureController::DidCollectGeometryForMagnificationGesture::name())
        return IPC::handleMessage<Messages::ViewGestureController::DidCollectGeometryForMagnificationGesture>(connection, decoder, this, &ViewGestureController::didCollectGeometryForMagnificationGesture);
#endif
#if !PLATFORM(IOS_FAMILY)
    if (decoder.messageName() == Messages::ViewGestureController::DidHitRenderTreeSizeThreshold::name())
        return IPC::handleMessage<Messages::ViewGestureController::DidHitRenderTreeSizeThreshold>(connection, decoder, this, &ViewGestureController::didHitRenderTreeSizeThreshold);
#endif
    UNUSED_PARAM(connection);
    UNUSED_PARAM(decoder);
#if ENABLE(IPC_TESTING_API)
    if (connection.ignoreInvalidMessageForTesting())
        return;
#endif // ENABLE(IPC_TESTING_API)
    ASSERT_NOT_REACHED_WITH_MESSAGE("Unhandled message %s to %" PRIu64, IPC::description(decoder.messageName()), decoder.destinationID());
}

} // namespace WebKit
