/*
 * Copyright (C) 2021 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#include "ArgumentCoders.h"
#include "Connection.h"
#include "DataReference.h"
#include "GeolocationIdentifier.h"
#include "IdentifierTypes.h"
#if PLATFORM(GTK) || PLATFORM(WPE)
#include "InputMethodState.h"
#endif
#if PLATFORM(COCOA)
#include "LayerHostingContext.h"
#endif
#include "MessageNames.h"
#include "PDFPluginIdentifier.h"
#include "ShareableBitmap.h"
#include "ShareableResource.h"
#include "SharedMemory.h"
#include "WebPageCreationParameters.h"
#include "WebPageProxyIdentifier.h"
#include "WebPageProxyMessagesReplies.h"
#include "WebPopupItem.h"
#include "WebSpeechSynthesisVoice.h"
#include "WebURLSchemeHandlerIdentifier.h"
#include <WebCore/ActivityState.h>
#include <WebCore/AutoplayEvent.h>
#include <WebCore/BackForwardItemIdentifier.h>
#include <WebCore/CaptureDevice.h>
#include <WebCore/Color.h>
#include <WebCore/ContactInfo.h>
#include <WebCore/DiagnosticLoggingClient.h>
#include <WebCore/DiagnosticLoggingDomain.h>
#include <WebCore/DictationContext.h>
#include <WebCore/DragActions.h>
#include <WebCore/FloatPoint.h>
#include <WebCore/FloatQuad.h>
#include <WebCore/FrameIdentifier.h>
#include <WebCore/FrameLoaderTypes.h>
#include <WebCore/HTMLModelElementCamera.h>
#include <WebCore/InspectorOverlay.h>
#include <WebCore/IntRect.h>
#include <WebCore/LayoutMilestone.h>
#include <WebCore/MediaControlsContextMenuItem.h>
#include <WebCore/MediaKeySystemRequestIdentifier.h>
#if ENABLE(WIRELESS_PLAYBACK_TARGET)
#include <WebCore/MediaPlaybackTargetContext.h>
#endif
#include <WebCore/MediaProducer.h>
#include <WebCore/MediaSessionIdentifier.h>
#include <WebCore/ModalContainerTypes.h>
#include <WebCore/PageIdentifier.h>
#include <WebCore/PermissionState.h>
#include <WebCore/PlaybackTargetClientContextIdentifier.h>
#include <WebCore/RectEdges.h>
#include <WebCore/ResourceError.h>
#include <WebCore/ResourceLoaderIdentifier.h>
#include <WebCore/SearchPopupMenu.h>
#include <WebCore/SerializedAttachmentData.h>
#include <WebCore/TextCheckerClient.h>
#include <WebCore/TextChecking.h>
#include <WebCore/TextIndicator.h>
#include <WebCore/TextManipulationController.h>
#include <WebCore/UserMediaRequestIdentifier.h>
#include <optional>
#include <utility>
#include <wtf/Expected.h>
#include <wtf/Forward.h>
#include <wtf/MachSendRight.h>
#include <wtf/OptionSet.h>
#include <wtf/Seconds.h>
#include <wtf/ThreadSafeRefCounted.h>
#include <wtf/Vector.h>
#include <wtf/text/WTFString.h>

namespace IPC {
class FormDataReference;
class SharedBufferCopy;
}

namespace Inspector {
enum class InspectorTargetType : uint8_t;
}

namespace WebCore {
class CertificateInfo;
class Color;
class ContentFilterUnblockHandler;
class Cursor;
class FloatPoint;
class FloatRect;
class FloatSize;
class IntPoint;
class IntRect;
class IntSize;
class LayoutPoint;
class PolicyCheckIdentifier;
class ResourceError;
class ResourceRequest;
class ResourceResponse;
class SelectionData;
class TextCheckingRequestData;
enum class LockBackForwardList : bool;
enum class ShouldSample : bool;
enum class WillContinueLoading : bool;
enum class AutoplayEvent : uint8_t;
enum class CookieConsentDecisionResult : uint8_t;
enum class DOMPasteAccessCategory : uint8_t;
enum class DOMPasteAccessResponse : uint8_t;
enum class DeviceOrientationOrMotionPermissionState : uint8_t;
enum class DragHandlingMethod : uint8_t;
enum class FrameLoadType : uint8_t;
enum class InputMode : uint8_t;
enum class ModalContainerDecision : uint8_t;
enum class MouseEventPolicy : uint8_t;
enum class RouteSharingPolicy : uint8_t;
enum class WebGLLoadPolicy : uint8_t;
struct AppHighlight;
struct ApplePayAMSUIRequest;
struct BackForwardItemIdentifier;
struct ClientOrigin;
struct ContactsRequestData;
struct ContentRuleListResults;
struct DataDetectorElementInfo;
struct DataListSuggestionInformation;
struct DateTimeChooserParameters;
struct DictionaryPopupInfo;
struct DragItem;
struct FileChooserSettings;
struct GlobalWindowIdentifier;
struct GrammarDetail;
struct HTMLModelElementCamera;
struct LinkIcon;
struct MediaStreamRequest;
struct MediaUsageInfo;
struct MockWebAuthenticationConfiguration;
struct PermissionDescriptor;
struct PromisedAttachmentInfo;
struct SecurityOriginData;
struct ShareDataWithParsedURL;
struct TextIndicatorData;
struct TextRecognitionResult;
struct TranslationContextMenuInfo;
struct ViewportAttributes;
struct WindowFeatures;
}

namespace WebKit {
class CallbackID;
class ContextMenuContextData;
class TouchBarMenuData;
class UserData;
enum class UndoOrRedo : bool;
struct BackForwardListItemState;
struct DataDetectionResult;
struct EditorState;
struct FocusedElementInformation;
struct FrameInfoData;
struct InteractionInformationAtPosition;
struct ModelIdentifier;
struct NavigationActionData;
struct PDFContextMenu;
struct PlatformPopupMenuData;
struct PolicyDecision;
struct TouchBarMenuItemData;
struct URLSchemeTaskParameters;
struct UserMessage;
struct WebAutocorrectionContext;
struct WebBackForwardListCounts;
struct WebHitTestResultData;
struct WebNavigationDataStore;
}

namespace Messages {
namespace WebPageProxy {

static inline IPC::ReceiverName messageReceiverName()
{
    return IPC::ReceiverName::WebPageProxy;
}

class CreateNewPage {
public:
    using Arguments = std::tuple<const WebKit::FrameInfoData&, const WebKit::WebPageProxyIdentifier&, const WebCore::ResourceRequest&, const WebCore::WindowFeatures&, const WebKit::NavigationActionData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_CreateNewPage; }
    static constexpr bool isSync = true;

    using DelayedReply = CreateNewPageDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const std::optional<WebCore::PageIdentifier>& newPageID, const std::optional<WebKit::WebPageCreationParameters>& newPageParameters);
    using Reply = std::tuple<std::optional<WebCore::PageIdentifier>&, std::optional<WebKit::WebPageCreationParameters>&>;
    using ReplyArguments = std::tuple<std::optional<WebCore::PageIdentifier>, std::optional<WebKit::WebPageCreationParameters>>;
    CreateNewPage(const WebKit::FrameInfoData& originatingFrameInfoData, const WebKit::WebPageProxyIdentifier& originatingPageID, const WebCore::ResourceRequest& request, const WebCore::WindowFeatures& windowFeatures, const WebKit::NavigationActionData& navigationActionData)
        : m_arguments(originatingFrameInfoData, originatingPageID, request, windowFeatures, navigationActionData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class ShowPage {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ShowPage; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class ClosePage {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ClosePage; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class RunJavaScriptAlert {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebKit::FrameInfoData&, const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RunJavaScriptAlert; }
    static constexpr bool isSync = true;

    using DelayedReply = RunJavaScriptAlertDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&);
    using Reply = std::tuple<>;
    using ReplyArguments = std::tuple<>;
    RunJavaScriptAlert(const WebCore::FrameIdentifier& frameID, const WebKit::FrameInfoData& frameInfo, const String& message)
        : m_arguments(frameID, frameInfo, message)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class RunJavaScriptConfirm {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebKit::FrameInfoData&, const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RunJavaScriptConfirm; }
    static constexpr bool isSync = true;

    using DelayedReply = RunJavaScriptConfirmDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, bool result);
    using Reply = std::tuple<bool&>;
    using ReplyArguments = std::tuple<bool>;
    RunJavaScriptConfirm(const WebCore::FrameIdentifier& frameID, const WebKit::FrameInfoData& frameInfo, const String& message)
        : m_arguments(frameID, frameInfo, message)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class RunJavaScriptPrompt {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebKit::FrameInfoData&, const String&, const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RunJavaScriptPrompt; }
    static constexpr bool isSync = true;

    using DelayedReply = RunJavaScriptPromptDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const String& result);
    using Reply = std::tuple<String&>;
    using ReplyArguments = std::tuple<String>;
    RunJavaScriptPrompt(const WebCore::FrameIdentifier& frameID, const WebKit::FrameInfoData& frameInfo, const String& message, const String& defaultValue)
        : m_arguments(frameID, frameInfo, message, defaultValue)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class MouseDidMoveOverElement {
public:
    using Arguments = std::tuple<const WebKit::WebHitTestResultData&, uint32_t, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_MouseDidMoveOverElement; }
    static constexpr bool isSync = false;

    MouseDidMoveOverElement(const WebKit::WebHitTestResultData& hitTestResultData, uint32_t modifiers, const WebKit::UserData& userData)
        : m_arguments(hitTestResultData, modifiers, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if ENABLE(WEBGL)
class WebGLPolicyForURL {
public:
    using Arguments = std::tuple<const URL&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_WebGLPolicyForURL; }
    static constexpr bool isSync = true;

    using DelayedReply = WebGLPolicyForURLDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, WebCore::WebGLLoadPolicy loadPolicy);
    using Reply = std::tuple<WebCore::WebGLLoadPolicy&>;
    using ReplyArguments = std::tuple<WebCore::WebGLLoadPolicy>;
    explicit WebGLPolicyForURL(const URL& url)
        : m_arguments(url)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(WEBGL)
class ResolveWebGLPolicyForURL {
public:
    using Arguments = std::tuple<const URL&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ResolveWebGLPolicyForURL; }
    static constexpr bool isSync = true;

    using DelayedReply = ResolveWebGLPolicyForURLDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, WebCore::WebGLLoadPolicy loadPolicy);
    using Reply = std::tuple<WebCore::WebGLLoadPolicy&>;
    using ReplyArguments = std::tuple<WebCore::WebGLLoadPolicy>;
    explicit ResolveWebGLPolicyForURL(const URL& url)
        : m_arguments(url)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class DidChangeViewportProperties {
public:
    using Arguments = std::tuple<const WebCore::ViewportAttributes&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidChangeViewportProperties; }
    static constexpr bool isSync = false;

    explicit DidChangeViewportProperties(const WebCore::ViewportAttributes& attributes)
        : m_arguments(attributes)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidReceiveEvent {
public:
    using Arguments = std::tuple<uint32_t, bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidReceiveEvent; }
    static constexpr bool isSync = false;

    DidReceiveEvent(uint32_t type, bool handled)
        : m_arguments(type, handled)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SetCursor {
public:
    using Arguments = std::tuple<const WebCore::Cursor&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetCursor; }
    static constexpr bool isSync = false;

    explicit SetCursor(const WebCore::Cursor& cursor)
        : m_arguments(cursor)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SetCursorHiddenUntilMouseMoves {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetCursorHiddenUntilMouseMoves; }
    static constexpr bool isSync = false;

    explicit SetCursorHiddenUntilMouseMoves(bool hiddenUntilMouseMoves)
        : m_arguments(hiddenUntilMouseMoves)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SetStatusText {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetStatusText; }
    static constexpr bool isSync = false;

    explicit SetStatusText(const String& statusText)
        : m_arguments(statusText)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SetFocus {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetFocus; }
    static constexpr bool isSync = false;

    explicit SetFocus(bool focused)
        : m_arguments(focused)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class TakeFocus {
public:
    using Arguments = std::tuple<uint8_t>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_TakeFocus; }
    static constexpr bool isSync = false;

    explicit TakeFocus(uint8_t direction)
        : m_arguments(direction)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class FocusedFrameChanged {
public:
    using Arguments = std::tuple<const std::optional<WebCore::FrameIdentifier>&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_FocusedFrameChanged; }
    static constexpr bool isSync = false;

    explicit FocusedFrameChanged(const std::optional<WebCore::FrameIdentifier>& frameID)
        : m_arguments(frameID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SetRenderTreeSize {
public:
    using Arguments = std::tuple<uint64_t>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetRenderTreeSize; }
    static constexpr bool isSync = false;

    explicit SetRenderTreeSize(uint64_t treeSize)
        : m_arguments(treeSize)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SetToolbarsAreVisible {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetToolbarsAreVisible; }
    static constexpr bool isSync = false;

    explicit SetToolbarsAreVisible(bool toolbarsAreVisible)
        : m_arguments(toolbarsAreVisible)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class GetToolbarsAreVisible {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_GetToolbarsAreVisible; }
    static constexpr bool isSync = true;

    using DelayedReply = GetToolbarsAreVisibleDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, bool toolbarsAreVisible);
    using Reply = std::tuple<bool&>;
    using ReplyArguments = std::tuple<bool>;
    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SetMenuBarIsVisible {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetMenuBarIsVisible; }
    static constexpr bool isSync = false;

    explicit SetMenuBarIsVisible(bool menuBarIsVisible)
        : m_arguments(menuBarIsVisible)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class GetMenuBarIsVisible {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_GetMenuBarIsVisible; }
    static constexpr bool isSync = true;

    using DelayedReply = GetMenuBarIsVisibleDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, bool menuBarIsVisible);
    using Reply = std::tuple<bool&>;
    using ReplyArguments = std::tuple<bool>;
    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SetStatusBarIsVisible {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetStatusBarIsVisible; }
    static constexpr bool isSync = false;

    explicit SetStatusBarIsVisible(bool statusBarIsVisible)
        : m_arguments(statusBarIsVisible)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class GetStatusBarIsVisible {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_GetStatusBarIsVisible; }
    static constexpr bool isSync = true;

    using DelayedReply = GetStatusBarIsVisibleDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, bool statusBarIsVisible);
    using Reply = std::tuple<bool&>;
    using ReplyArguments = std::tuple<bool>;
    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SetIsResizable {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetIsResizable; }
    static constexpr bool isSync = false;

    explicit SetIsResizable(bool isResizable)
        : m_arguments(isResizable)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SetWindowFrame {
public:
    using Arguments = std::tuple<const WebCore::FloatRect&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetWindowFrame; }
    static constexpr bool isSync = false;

    explicit SetWindowFrame(const WebCore::FloatRect& windowFrame)
        : m_arguments(windowFrame)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class GetWindowFrame {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_GetWindowFrame; }
    static constexpr bool isSync = true;

    using DelayedReply = GetWindowFrameDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const WebCore::FloatRect& windowFrame);
    using Reply = std::tuple<WebCore::FloatRect&>;
    using ReplyArguments = std::tuple<WebCore::FloatRect>;
    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class ScreenToRootView {
public:
    using Arguments = std::tuple<const WebCore::IntPoint&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ScreenToRootView; }
    static constexpr bool isSync = true;

    using DelayedReply = ScreenToRootViewDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const WebCore::IntPoint& windowPoint);
    using Reply = std::tuple<WebCore::IntPoint&>;
    using ReplyArguments = std::tuple<WebCore::IntPoint>;
    explicit ScreenToRootView(const WebCore::IntPoint& screenPoint)
        : m_arguments(screenPoint)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class RootViewToScreen {
public:
    using Arguments = std::tuple<const WebCore::IntRect&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RootViewToScreen; }
    static constexpr bool isSync = true;

    using DelayedReply = RootViewToScreenDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const WebCore::IntRect& screenFrame);
    using Reply = std::tuple<WebCore::IntRect&>;
    using ReplyArguments = std::tuple<WebCore::IntRect>;
    explicit RootViewToScreen(const WebCore::IntRect& rect)
        : m_arguments(rect)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class AccessibilityScreenToRootView {
public:
    using Arguments = std::tuple<const WebCore::IntPoint&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_AccessibilityScreenToRootView; }
    static constexpr bool isSync = true;

    using DelayedReply = AccessibilityScreenToRootViewDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const WebCore::IntPoint& windowPoint);
    using Reply = std::tuple<WebCore::IntPoint&>;
    using ReplyArguments = std::tuple<WebCore::IntPoint>;
    explicit AccessibilityScreenToRootView(const WebCore::IntPoint& screenPoint)
        : m_arguments(screenPoint)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class RootViewToAccessibilityScreen {
public:
    using Arguments = std::tuple<const WebCore::IntRect&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RootViewToAccessibilityScreen; }
    static constexpr bool isSync = true;

    using DelayedReply = RootViewToAccessibilityScreenDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const WebCore::IntRect& screenFrame);
    using Reply = std::tuple<WebCore::IntRect&>;
    using ReplyArguments = std::tuple<WebCore::IntRect>;
    explicit RootViewToAccessibilityScreen(const WebCore::IntRect& rect)
        : m_arguments(rect)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if PLATFORM(COCOA) || PLATFORM(GTK)
class ShowValidationMessage {
public:
    using Arguments = std::tuple<const WebCore::IntRect&, const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ShowValidationMessage; }
    static constexpr bool isSync = false;

    ShowValidationMessage(const WebCore::IntRect& anchorRect, const String& message)
        : m_arguments(anchorRect, message)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(COCOA) || PLATFORM(GTK)
class HideValidationMessage {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_HideValidationMessage; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class RunBeforeUnloadConfirmPanel {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebKit::FrameInfoData&, const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RunBeforeUnloadConfirmPanel; }
    static constexpr bool isSync = true;

    using DelayedReply = RunBeforeUnloadConfirmPanelDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, bool shouldClose);
    using Reply = std::tuple<bool&>;
    using ReplyArguments = std::tuple<bool>;
    RunBeforeUnloadConfirmPanel(const WebCore::FrameIdentifier& frameID, const WebKit::FrameInfoData& frameInfo, const String& message)
        : m_arguments(frameID, frameInfo, message)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class PageDidScroll {
public:
    using Arguments = std::tuple<const WebCore::IntPoint&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_PageDidScroll; }
    static constexpr bool isSync = false;

    explicit PageDidScroll(const WebCore::IntPoint& scrollPosition)
        : m_arguments(scrollPosition)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SetHasActiveAnimatedScrolls {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetHasActiveAnimatedScrolls; }
    static constexpr bool isSync = false;

    explicit SetHasActiveAnimatedScrolls(bool hasActiveAnimatedScrolls)
        : m_arguments(hasActiveAnimatedScrolls)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class RunOpenPanel {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebKit::FrameInfoData&, const WebCore::FileChooserSettings&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RunOpenPanel; }
    static constexpr bool isSync = false;

    RunOpenPanel(const WebCore::FrameIdentifier& frameID, const WebKit::FrameInfoData& frameInfo, const WebCore::FileChooserSettings& parameters)
        : m_arguments(frameID, frameInfo, parameters)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class ShowShareSheet {
public:
    using Arguments = std::tuple<const WebCore::ShareDataWithParsedURL&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ShowShareSheet; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(bool&&)>&&);
    static void cancelReply(CompletionHandler<void(bool&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_ShowShareSheetReply; }
    using AsyncReply = ShowShareSheetAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, bool granted);
    using Reply = std::tuple<bool&>;
    using ReplyArguments = std::tuple<bool>;
    explicit ShowShareSheet(const WebCore::ShareDataWithParsedURL& shareData)
        : m_arguments(shareData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class ShowContactPicker {
public:
    using Arguments = std::tuple<const WebCore::ContactsRequestData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ShowContactPicker; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(std::optional<Vector<WebCore::ContactInfo>>&&)>&&);
    static void cancelReply(CompletionHandler<void(std::optional<Vector<WebCore::ContactInfo>>&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_ShowContactPickerReply; }
    using AsyncReply = ShowContactPickerAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const std::optional<Vector<WebCore::ContactInfo>>& info);
    using Reply = std::tuple<std::optional<Vector<WebCore::ContactInfo>>&>;
    using ReplyArguments = std::tuple<std::optional<Vector<WebCore::ContactInfo>>>;
    explicit ShowContactPicker(const WebCore::ContactsRequestData& requestData)
        : m_arguments(requestData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class PrintFrame {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const String&, const WebCore::FloatSize&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_PrintFrame; }
    static constexpr bool isSync = true;

    using DelayedReply = PrintFrameDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&);
    using Reply = std::tuple<>;
    using ReplyArguments = std::tuple<>;
    PrintFrame(const WebCore::FrameIdentifier& frameID, const String& title, const WebCore::FloatSize& pdfFirstPageSize)
        : m_arguments(frameID, title, pdfFirstPageSize)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class RunModal {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RunModal; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class NotifyScrollerThumbIsVisibleInRect {
public:
    using Arguments = std::tuple<const WebCore::IntRect&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_NotifyScrollerThumbIsVisibleInRect; }
    static constexpr bool isSync = false;

    explicit NotifyScrollerThumbIsVisibleInRect(const WebCore::IntRect& scrollerThumb)
        : m_arguments(scrollerThumb)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class RecommendedScrollbarStyleDidChange {
public:
    using Arguments = std::tuple<int32_t>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RecommendedScrollbarStyleDidChange; }
    static constexpr bool isSync = false;

    explicit RecommendedScrollbarStyleDidChange(int32_t newStyle)
        : m_arguments(newStyle)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidChangeScrollbarsForMainFrame {
public:
    using Arguments = std::tuple<bool, bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidChangeScrollbarsForMainFrame; }
    static constexpr bool isSync = false;

    DidChangeScrollbarsForMainFrame(bool hasHorizontalScrollbar, bool hasVerticalScrollbar)
        : m_arguments(hasHorizontalScrollbar, hasVerticalScrollbar)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidChangeScrollOffsetPinningForMainFrame {
public:
    using Arguments = std::tuple<const WebCore::RectEdges<bool>&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidChangeScrollOffsetPinningForMainFrame; }
    static constexpr bool isSync = false;

    explicit DidChangeScrollOffsetPinningForMainFrame(const WebCore::RectEdges<bool>& pinnedState)
        : m_arguments(pinnedState)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidChangePageCount {
public:
    using Arguments = std::tuple<const unsigned&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidChangePageCount; }
    static constexpr bool isSync = false;

    explicit DidChangePageCount(const unsigned& pageCount)
        : m_arguments(pageCount)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class ThemeColorChanged {
public:
    using Arguments = std::tuple<const WebCore::Color&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ThemeColorChanged; }
    static constexpr bool isSync = false;

    explicit ThemeColorChanged(const WebCore::Color& themeColor)
        : m_arguments(themeColor)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class PageExtendedBackgroundColorDidChange {
public:
    using Arguments = std::tuple<const WebCore::Color&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_PageExtendedBackgroundColorDidChange; }
    static constexpr bool isSync = false;

    explicit PageExtendedBackgroundColorDidChange(const WebCore::Color& backgroundColor)
        : m_arguments(backgroundColor)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SampledPageTopColorChanged {
public:
    using Arguments = std::tuple<const WebCore::Color&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SampledPageTopColorChanged; }
    static constexpr bool isSync = false;

    explicit SampledPageTopColorChanged(const WebCore::Color& sampledPageTopColor)
        : m_arguments(sampledPageTopColor)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SetCanShortCircuitHorizontalWheelEvents {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetCanShortCircuitHorizontalWheelEvents; }
    static constexpr bool isSync = false;

    explicit SetCanShortCircuitHorizontalWheelEvents(bool canShortCircuitHorizontalWheelEvents)
        : m_arguments(canShortCircuitHorizontalWheelEvents)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidChangeContentSize {
public:
    using Arguments = std::tuple<const WebCore::IntSize&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidChangeContentSize; }
    static constexpr bool isSync = false;

    explicit DidChangeContentSize(const WebCore::IntSize& newSize)
        : m_arguments(newSize)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidChangeIntrinsicContentSize {
public:
    using Arguments = std::tuple<const WebCore::IntSize&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidChangeIntrinsicContentSize; }
    static constexpr bool isSync = false;

    explicit DidChangeIntrinsicContentSize(const WebCore::IntSize& newIntrinsicContentSize)
        : m_arguments(newIntrinsicContentSize)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if ENABLE(INPUT_TYPE_COLOR)
class ShowColorPicker {
public:
    using Arguments = std::tuple<const WebCore::Color&, const WebCore::IntRect&, const Vector<WebCore::Color>&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ShowColorPicker; }
    static constexpr bool isSync = false;

    ShowColorPicker(const WebCore::Color& initialColor, const WebCore::IntRect& elementRect, const Vector<WebCore::Color>& suggestions)
        : m_arguments(initialColor, elementRect, suggestions)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(INPUT_TYPE_COLOR)
class SetColorPickerColor {
public:
    using Arguments = std::tuple<const WebCore::Color&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetColorPickerColor; }
    static constexpr bool isSync = false;

    explicit SetColorPickerColor(const WebCore::Color& color)
        : m_arguments(color)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(INPUT_TYPE_COLOR)
class EndColorPicker {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_EndColorPicker; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(DATALIST_ELEMENT)
class ShowDataListSuggestions {
public:
    using Arguments = std::tuple<const WebCore::DataListSuggestionInformation&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ShowDataListSuggestions; }
    static constexpr bool isSync = false;

    explicit ShowDataListSuggestions(const WebCore::DataListSuggestionInformation& suggestionInformation)
        : m_arguments(suggestionInformation)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(DATALIST_ELEMENT)
class HandleKeydownInDataList {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_HandleKeydownInDataList; }
    static constexpr bool isSync = false;

    explicit HandleKeydownInDataList(const String& key)
        : m_arguments(key)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(DATALIST_ELEMENT)
class EndDataListSuggestions {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_EndDataListSuggestions; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(DATE_AND_TIME_INPUT_TYPES)
class ShowDateTimePicker {
public:
    using Arguments = std::tuple<const WebCore::DateTimeChooserParameters&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ShowDateTimePicker; }
    static constexpr bool isSync = false;

    explicit ShowDateTimePicker(const WebCore::DateTimeChooserParameters& params)
        : m_arguments(params)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(DATE_AND_TIME_INPUT_TYPES)
class EndDateTimePicker {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_EndDateTimePicker; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class DecidePolicyForResponse {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebKit::FrameInfoData&, const WebCore::PolicyCheckIdentifier&, uint64_t, const WebCore::ResourceResponse&, const WebCore::ResourceRequest&, bool, const String&, bool, uint64_t, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DecidePolicyForResponse; }
    static constexpr bool isSync = false;

    DecidePolicyForResponse(const WebCore::FrameIdentifier& frameID, const WebKit::FrameInfoData& frameInfo, const WebCore::PolicyCheckIdentifier& policyCheckIdentifier, uint64_t navigationID, const WebCore::ResourceResponse& response, const WebCore::ResourceRequest& request, bool canShowMIMEType, const String& downloadAttribute, bool wasAllowedByInjectedBundle, uint64_t listenerID, const WebKit::UserData& userData)
        : m_arguments(frameID, frameInfo, policyCheckIdentifier, navigationID, response, request, canShowMIMEType, downloadAttribute, wasAllowedByInjectedBundle, listenerID, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DecidePolicyForNavigationActionAsync {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebKit::FrameInfoData&, const WebCore::PolicyCheckIdentifier&, uint64_t, const WebKit::NavigationActionData&, const WebKit::FrameInfoData&, const std::optional<WebKit::WebPageProxyIdentifier>&, const WebCore::ResourceRequest&, const WebCore::ResourceRequest&, const IPC::FormDataReference&, const WebCore::ResourceResponse&, const WebKit::UserData&, uint64_t>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DecidePolicyForNavigationActionAsync; }
    static constexpr bool isSync = false;

    DecidePolicyForNavigationActionAsync(const WebCore::FrameIdentifier& frameID, const WebKit::FrameInfoData& frameInfo, const WebCore::PolicyCheckIdentifier& policyCheckIdentifier, uint64_t navigationID, const WebKit::NavigationActionData& navigationActionData, const WebKit::FrameInfoData& originatingFrameInfoData, const std::optional<WebKit::WebPageProxyIdentifier>& originatingPageID, const WebCore::ResourceRequest& originalRequest, const WebCore::ResourceRequest& request, const IPC::FormDataReference& requestBody, const WebCore::ResourceResponse& redirectResponse, const WebKit::UserData& userData, uint64_t listenerID)
        : m_arguments(frameID, frameInfo, policyCheckIdentifier, navigationID, navigationActionData, originatingFrameInfoData, originatingPageID, originalRequest, request, requestBody, redirectResponse, userData, listenerID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DecidePolicyForNavigationActionSync {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, bool, const WebKit::FrameInfoData&, const WebCore::PolicyCheckIdentifier&, uint64_t, const WebKit::NavigationActionData&, const WebKit::FrameInfoData&, const std::optional<WebKit::WebPageProxyIdentifier>&, const WebCore::ResourceRequest&, const WebCore::ResourceRequest&, const IPC::FormDataReference&, const WebCore::ResourceResponse&, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DecidePolicyForNavigationActionSync; }
    static constexpr bool isSync = true;

    using DelayedReply = DecidePolicyForNavigationActionSyncDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const WebKit::PolicyDecision& PolicyDecision);
    using Reply = std::tuple<WebKit::PolicyDecision&>;
    using ReplyArguments = std::tuple<WebKit::PolicyDecision>;
    DecidePolicyForNavigationActionSync(const WebCore::FrameIdentifier& frameID, bool isMainFrame, const WebKit::FrameInfoData& frameInfo, const WebCore::PolicyCheckIdentifier& policyCheckIdentifier, uint64_t navigationID, const WebKit::NavigationActionData& navigationActionData, const WebKit::FrameInfoData& originatingFrameInfoData, const std::optional<WebKit::WebPageProxyIdentifier>& originatingPageID, const WebCore::ResourceRequest& originalRequest, const WebCore::ResourceRequest& request, const IPC::FormDataReference& requestBody, const WebCore::ResourceResponse& redirectResponse, const WebKit::UserData& userData)
        : m_arguments(frameID, isMainFrame, frameInfo, policyCheckIdentifier, navigationID, navigationActionData, originatingFrameInfoData, originatingPageID, originalRequest, request, requestBody, redirectResponse, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DecidePolicyForNewWindowAction {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebKit::FrameInfoData&, const WebCore::PolicyCheckIdentifier&, const WebKit::NavigationActionData&, const WebCore::ResourceRequest&, const String&, uint64_t, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DecidePolicyForNewWindowAction; }
    static constexpr bool isSync = false;

    DecidePolicyForNewWindowAction(const WebCore::FrameIdentifier& frameID, const WebKit::FrameInfoData& frameInfo, const WebCore::PolicyCheckIdentifier& policyCheckIdentifier, const WebKit::NavigationActionData& navigationActionData, const WebCore::ResourceRequest& request, const String& frameName, uint64_t listenerID, const WebKit::UserData& userData)
        : m_arguments(frameID, frameInfo, policyCheckIdentifier, navigationActionData, request, frameName, listenerID, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class UnableToImplementPolicy {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebCore::ResourceError&, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_UnableToImplementPolicy; }
    static constexpr bool isSync = false;

    UnableToImplementPolicy(const WebCore::FrameIdentifier& frameID, const WebCore::ResourceError& error, const WebKit::UserData& userData)
        : m_arguments(frameID, error, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidChangeProgress {
public:
    using Arguments = std::tuple<double>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidChangeProgress; }
    static constexpr bool isSync = false;

    explicit DidChangeProgress(double value)
        : m_arguments(value)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidFinishProgress {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidFinishProgress; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidStartProgress {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidStartProgress; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SetNetworkRequestsInProgress {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetNetworkRequestsInProgress; }
    static constexpr bool isSync = false;

    explicit SetNetworkRequestsInProgress(bool networkRequestsInProgress)
        : m_arguments(networkRequestsInProgress)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidCreateMainFrame {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidCreateMainFrame; }
    static constexpr bool isSync = false;

    explicit DidCreateMainFrame(const WebCore::FrameIdentifier& frameID)
        : m_arguments(frameID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidCreateSubframe {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidCreateSubframe; }
    static constexpr bool isSync = false;

    explicit DidCreateSubframe(const WebCore::FrameIdentifier& frameID)
        : m_arguments(frameID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidCreateWindow {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebCore::GlobalWindowIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidCreateWindow; }
    static constexpr bool isSync = false;

    DidCreateWindow(const WebCore::FrameIdentifier& frameID, const WebCore::GlobalWindowIdentifier& windowIdentifier)
        : m_arguments(frameID, windowIdentifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidStartProvisionalLoadForFrame {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebKit::FrameInfoData&, const WebCore::ResourceRequest&, uint64_t, const URL&, const URL&, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidStartProvisionalLoadForFrame; }
    static constexpr bool isSync = false;

    DidStartProvisionalLoadForFrame(const WebCore::FrameIdentifier& frameID, const WebKit::FrameInfoData& frameInfo, const WebCore::ResourceRequest& request, uint64_t navigationID, const URL& url, const URL& unreachableURL, const WebKit::UserData& userData)
        : m_arguments(frameID, frameInfo, request, navigationID, url, unreachableURL, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidReceiveServerRedirectForProvisionalLoadForFrame {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, uint64_t, const WebCore::ResourceRequest&, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidReceiveServerRedirectForProvisionalLoadForFrame; }
    static constexpr bool isSync = false;

    DidReceiveServerRedirectForProvisionalLoadForFrame(const WebCore::FrameIdentifier& frameID, uint64_t navigationID, const WebCore::ResourceRequest& request, const WebKit::UserData& userData)
        : m_arguments(frameID, navigationID, request, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class WillPerformClientRedirectForFrame {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const String&, double, WebCore::LockBackForwardList>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_WillPerformClientRedirectForFrame; }
    static constexpr bool isSync = false;

    WillPerformClientRedirectForFrame(const WebCore::FrameIdentifier& frameID, const String& url, double delay, WebCore::LockBackForwardList lockBackForwardList)
        : m_arguments(frameID, url, delay, lockBackForwardList)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidCancelClientRedirectForFrame {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidCancelClientRedirectForFrame; }
    static constexpr bool isSync = false;

    explicit DidCancelClientRedirectForFrame(const WebCore::FrameIdentifier& frameID)
        : m_arguments(frameID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidChangeProvisionalURLForFrame {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, uint64_t, const URL&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidChangeProvisionalURLForFrame; }
    static constexpr bool isSync = false;

    DidChangeProvisionalURLForFrame(const WebCore::FrameIdentifier& frameID, uint64_t navigationID, const URL& url)
        : m_arguments(frameID, navigationID, url)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidFailProvisionalLoadForFrame {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebKit::FrameInfoData&, const WebCore::ResourceRequest&, uint64_t, const String&, const WebCore::ResourceError&, WebCore::WillContinueLoading, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidFailProvisionalLoadForFrame; }
    static constexpr bool isSync = false;

    DidFailProvisionalLoadForFrame(const WebCore::FrameIdentifier& frameID, const WebKit::FrameInfoData& frameInfo, const WebCore::ResourceRequest& request, uint64_t navigationID, const String& provisionalURL, const WebCore::ResourceError& error, WebCore::WillContinueLoading willContinueLoading, const WebKit::UserData& userData)
        : m_arguments(frameID, frameInfo, request, navigationID, provisionalURL, error, willContinueLoading, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidCommitLoadForFrame {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebKit::FrameInfoData&, const WebCore::ResourceRequest&, uint64_t, const String&, bool, WebCore::FrameLoadType, const WebCore::CertificateInfo&, bool, bool, const std::optional<WebCore::HasInsecureContent>&, WebCore::MouseEventPolicy, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidCommitLoadForFrame; }
    static constexpr bool isSync = false;

    DidCommitLoadForFrame(const WebCore::FrameIdentifier& frameID, const WebKit::FrameInfoData& frameInfo, const WebCore::ResourceRequest& request, uint64_t navigationID, const String& mimeType, bool hasCustomContentProvider, WebCore::FrameLoadType loadType, const WebCore::CertificateInfo& certificateInfo, bool usedLegacyTLS, bool containsPluginDocument, const std::optional<WebCore::HasInsecureContent>& forcedHasInsecureContent, WebCore::MouseEventPolicy mouseEventPolicy, const WebKit::UserData& userData)
        : m_arguments(frameID, frameInfo, request, navigationID, mimeType, hasCustomContentProvider, loadType, certificateInfo, usedLegacyTLS, containsPluginDocument, forcedHasInsecureContent, mouseEventPolicy, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidFailLoadForFrame {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebKit::FrameInfoData&, const WebCore::ResourceRequest&, uint64_t, const WebCore::ResourceError&, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidFailLoadForFrame; }
    static constexpr bool isSync = false;

    DidFailLoadForFrame(const WebCore::FrameIdentifier& frameID, const WebKit::FrameInfoData& frameInfo, const WebCore::ResourceRequest& request, uint64_t navigationID, const WebCore::ResourceError& error, const WebKit::UserData& userData)
        : m_arguments(frameID, frameInfo, request, navigationID, error, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidFinishDocumentLoadForFrame {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, uint64_t, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidFinishDocumentLoadForFrame; }
    static constexpr bool isSync = false;

    DidFinishDocumentLoadForFrame(const WebCore::FrameIdentifier& frameID, uint64_t navigationID, const WebKit::UserData& userData)
        : m_arguments(frameID, navigationID, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidFinishLoadForFrame {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebKit::FrameInfoData&, const WebCore::ResourceRequest&, uint64_t, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidFinishLoadForFrame; }
    static constexpr bool isSync = false;

    DidFinishLoadForFrame(const WebCore::FrameIdentifier& frameID, const WebKit::FrameInfoData& frameInfo, const WebCore::ResourceRequest& request, uint64_t navigationID, const WebKit::UserData& userData)
        : m_arguments(frameID, frameInfo, request, navigationID, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidFirstLayoutForFrame {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidFirstLayoutForFrame; }
    static constexpr bool isSync = false;

    DidFirstLayoutForFrame(const WebCore::FrameIdentifier& frameID, const WebKit::UserData& userData)
        : m_arguments(frameID, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidFirstVisuallyNonEmptyLayoutForFrame {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidFirstVisuallyNonEmptyLayoutForFrame; }
    static constexpr bool isSync = false;

    DidFirstVisuallyNonEmptyLayoutForFrame(const WebCore::FrameIdentifier& frameID, const WebKit::UserData& userData)
        : m_arguments(frameID, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidReachLayoutMilestone {
public:
    using Arguments = std::tuple<const OptionSet<WebCore::LayoutMilestone>&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidReachLayoutMilestone; }
    static constexpr bool isSync = false;

    explicit DidReachLayoutMilestone(const OptionSet<WebCore::LayoutMilestone>& layoutMilestones)
        : m_arguments(layoutMilestones)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidReceiveTitleForFrame {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const String&, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidReceiveTitleForFrame; }
    static constexpr bool isSync = false;

    DidReceiveTitleForFrame(const WebCore::FrameIdentifier& frameID, const String& title, const WebKit::UserData& userData)
        : m_arguments(frameID, title, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidDisplayInsecureContentForFrame {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidDisplayInsecureContentForFrame; }
    static constexpr bool isSync = false;

    DidDisplayInsecureContentForFrame(const WebCore::FrameIdentifier& frameID, const WebKit::UserData& userData)
        : m_arguments(frameID, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidRunInsecureContentForFrame {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidRunInsecureContentForFrame; }
    static constexpr bool isSync = false;

    DidRunInsecureContentForFrame(const WebCore::FrameIdentifier& frameID, const WebKit::UserData& userData)
        : m_arguments(frameID, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidDetectXSSForFrame {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidDetectXSSForFrame; }
    static constexpr bool isSync = false;

    DidDetectXSSForFrame(const WebCore::FrameIdentifier& frameID, const WebKit::UserData& userData)
        : m_arguments(frameID, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidSameDocumentNavigationForFrame {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, uint64_t, uint32_t, const URL&, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidSameDocumentNavigationForFrame; }
    static constexpr bool isSync = false;

    DidSameDocumentNavigationForFrame(const WebCore::FrameIdentifier& frameID, uint64_t navigationID, uint32_t type, const URL& url, const WebKit::UserData& userData)
        : m_arguments(frameID, navigationID, type, url, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidChangeMainDocument {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidChangeMainDocument; }
    static constexpr bool isSync = false;

    explicit DidChangeMainDocument(const WebCore::FrameIdentifier& frameID)
        : m_arguments(frameID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidExplicitOpenForFrame {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const URL&, const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidExplicitOpenForFrame; }
    static constexpr bool isSync = false;

    DidExplicitOpenForFrame(const WebCore::FrameIdentifier& frameID, const URL& url, const String& mimeType)
        : m_arguments(frameID, url, mimeType)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidDestroyNavigation {
public:
    using Arguments = std::tuple<uint64_t>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidDestroyNavigation; }
    static constexpr bool isSync = false;

    explicit DidDestroyNavigation(uint64_t navigationID)
        : m_arguments(navigationID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class MainFramePluginHandlesPageScaleGestureDidChange {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_MainFramePluginHandlesPageScaleGestureDidChange; }
    static constexpr bool isSync = false;

    explicit MainFramePluginHandlesPageScaleGestureDidChange(bool mainFramePluginHandlesPageScaleGesture)
        : m_arguments(mainFramePluginHandlesPageScaleGesture)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidNavigateWithNavigationData {
public:
    using Arguments = std::tuple<const WebKit::WebNavigationDataStore&, const WebCore::FrameIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidNavigateWithNavigationData; }
    static constexpr bool isSync = false;

    DidNavigateWithNavigationData(const WebKit::WebNavigationDataStore& store, const WebCore::FrameIdentifier& frameID)
        : m_arguments(store, frameID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidPerformClientRedirect {
public:
    using Arguments = std::tuple<const String&, const String&, const WebCore::FrameIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidPerformClientRedirect; }
    static constexpr bool isSync = false;

    DidPerformClientRedirect(const String& sourceURLString, const String& destinationURLString, const WebCore::FrameIdentifier& frameID)
        : m_arguments(sourceURLString, destinationURLString, frameID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidPerformServerRedirect {
public:
    using Arguments = std::tuple<const String&, const String&, const WebCore::FrameIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidPerformServerRedirect; }
    static constexpr bool isSync = false;

    DidPerformServerRedirect(const String& sourceURLString, const String& destinationURLString, const WebCore::FrameIdentifier& frameID)
        : m_arguments(sourceURLString, destinationURLString, frameID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidUpdateHistoryTitle {
public:
    using Arguments = std::tuple<const String&, const String&, const WebCore::FrameIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidUpdateHistoryTitle; }
    static constexpr bool isSync = false;

    DidUpdateHistoryTitle(const String& title, const String& url, const WebCore::FrameIdentifier& frameID)
        : m_arguments(title, url, frameID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidFinishLoadingDataForCustomContentProvider {
public:
    using Arguments = std::tuple<const String&, const IPC::DataReference&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidFinishLoadingDataForCustomContentProvider; }
    static constexpr bool isSync = false;

    DidFinishLoadingDataForCustomContentProvider(const String& suggestedFilename, const IPC::DataReference& data)
        : m_arguments(suggestedFilename, data)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class WillSubmitForm {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebCore::FrameIdentifier&, const Vector<std::pair<String, String>>&, const WebKit::FormSubmitListenerIdentifier&, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_WillSubmitForm; }
    static constexpr bool isSync = false;

    WillSubmitForm(const WebCore::FrameIdentifier& frameID, const WebCore::FrameIdentifier& sourceFrameID, const Vector<std::pair<String, String>>& textFieldValues, const WebKit::FormSubmitListenerIdentifier& listenerID, const WebKit::UserData& userData)
        : m_arguments(frameID, sourceFrameID, textFieldValues, listenerID, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if PLATFORM(IOS_FAMILY)
class InterpretKeyEvent {
public:
    using Arguments = std::tuple<const WebKit::EditorState&, bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_InterpretKeyEvent; }
    static constexpr bool isSync = true;

    using DelayedReply = InterpretKeyEventDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, bool handled);
    using Reply = std::tuple<bool&>;
    using ReplyArguments = std::tuple<bool>;
    InterpretKeyEvent(const WebKit::EditorState& state, bool isCharEvent)
        : m_arguments(state, isCharEvent)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class DidReceivePositionInformation {
public:
    using Arguments = std::tuple<const WebKit::InteractionInformationAtPosition&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidReceivePositionInformation; }
    static constexpr bool isSync = false;

    explicit DidReceivePositionInformation(const WebKit::InteractionInformationAtPosition& information)
        : m_arguments(information)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class SaveImageToLibrary {
public:
    using Arguments = std::tuple<const WebKit::SharedMemory::IPCHandle&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SaveImageToLibrary; }
    static constexpr bool isSync = false;

    explicit SaveImageToLibrary(const WebKit::SharedMemory::IPCHandle& handle)
        : m_arguments(handle)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class ShowPlaybackTargetPicker {
public:
    using Arguments = std::tuple<bool, const WebCore::IntRect&, WebCore::RouteSharingPolicy, const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ShowPlaybackTargetPicker; }
    static constexpr bool isSync = false;

    ShowPlaybackTargetPicker(bool hasVideo, const WebCore::IntRect& elementRect, WebCore::RouteSharingPolicy policy, const String& routingContextUID)
        : m_arguments(hasVideo, elementRect, policy, routingContextUID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class CommitPotentialTapFailed {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_CommitPotentialTapFailed; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class DidNotHandleTapAsClick {
public:
    using Arguments = std::tuple<const WebCore::IntPoint&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidNotHandleTapAsClick; }
    static constexpr bool isSync = false;

    explicit DidNotHandleTapAsClick(const WebCore::IntPoint& point)
        : m_arguments(point)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class DidCompleteSyntheticClick {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidCompleteSyntheticClick; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class DisableDoubleTapGesturesDuringTapIfNecessary {
public:
    using Arguments = std::tuple<const WebKit::TapIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DisableDoubleTapGesturesDuringTapIfNecessary; }
    static constexpr bool isSync = false;

    explicit DisableDoubleTapGesturesDuringTapIfNecessary(const WebKit::TapIdentifier& requestID)
        : m_arguments(requestID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class HandleSmartMagnificationInformationForPotentialTap {
public:
    using Arguments = std::tuple<const WebKit::TapIdentifier&, const WebCore::FloatRect&, bool, double, double, bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_HandleSmartMagnificationInformationForPotentialTap; }
    static constexpr bool isSync = false;

    HandleSmartMagnificationInformationForPotentialTap(const WebKit::TapIdentifier& requestID, const WebCore::FloatRect& renderRect, bool fitEntireRect, double viewportMinimumScale, double viewportMaximumScale, bool nodeIsRootLevel)
        : m_arguments(requestID, renderRect, fitEntireRect, viewportMinimumScale, viewportMaximumScale, nodeIsRootLevel)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(DATA_DETECTION)
class SetDataDetectionResult {
public:
    using Arguments = std::tuple<const WebKit::DataDetectionResult&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetDataDetectionResult; }
    static constexpr bool isSync = false;

    explicit SetDataDetectionResult(const WebKit::DataDetectionResult& dataDetectionResult)
        : m_arguments(dataDetectionResult)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(DATA_DETECTION)
class HandleClickForDataDetectionResult {
public:
    using Arguments = std::tuple<const WebCore::DataDetectorElementInfo&, const WebCore::IntPoint&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_HandleClickForDataDetectionResult; }
    static constexpr bool isSync = false;

    HandleClickForDataDetectionResult(const WebCore::DataDetectorElementInfo& info, const WebCore::IntPoint& location)
        : m_arguments(info, location)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class PageScaleFactorDidChange {
public:
    using Arguments = std::tuple<double>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_PageScaleFactorDidChange; }
    static constexpr bool isSync = false;

    explicit PageScaleFactorDidChange(double scaleFactor)
        : m_arguments(scaleFactor)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class PluginScaleFactorDidChange {
public:
    using Arguments = std::tuple<double>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_PluginScaleFactorDidChange; }
    static constexpr bool isSync = false;

    explicit PluginScaleFactorDidChange(double zoomFactor)
        : m_arguments(zoomFactor)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class PluginZoomFactorDidChange {
public:
    using Arguments = std::tuple<double>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_PluginZoomFactorDidChange; }
    static constexpr bool isSync = false;

    explicit PluginZoomFactorDidChange(double zoomFactor)
        : m_arguments(zoomFactor)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if PLATFORM(GTK) || PLATFORM(WPE)
class BindAccessibilityTree {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_BindAccessibilityTree; }
    static constexpr bool isSync = false;

    explicit BindAccessibilityTree(const String& plugID)
        : m_arguments(plugID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(GTK) || PLATFORM(WPE)
class SetInputMethodState {
public:
    using Arguments = std::tuple<const std::optional<WebKit::InputMethodState>&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetInputMethodState; }
    static constexpr bool isSync = false;

    explicit SetInputMethodState(const std::optional<WebKit::InputMethodState>& state)
        : m_arguments(state)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class BackForwardAddItem {
public:
    using Arguments = std::tuple<const WebKit::BackForwardListItemState&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_BackForwardAddItem; }
    static constexpr bool isSync = false;

    explicit BackForwardAddItem(const WebKit::BackForwardListItemState& itemState)
        : m_arguments(itemState)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class BackForwardGoToItem {
public:
    using Arguments = std::tuple<const WebCore::BackForwardItemIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_BackForwardGoToItem; }
    static constexpr bool isSync = true;

    using DelayedReply = BackForwardGoToItemDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const WebKit::WebBackForwardListCounts& counts);
    using Reply = std::tuple<WebKit::WebBackForwardListCounts&>;
    using ReplyArguments = std::tuple<WebKit::WebBackForwardListCounts>;
    explicit BackForwardGoToItem(const WebCore::BackForwardItemIdentifier& itemID)
        : m_arguments(itemID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class BackForwardItemAtIndex {
public:
    using Arguments = std::tuple<int32_t>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_BackForwardItemAtIndex; }
    static constexpr bool isSync = true;

    using DelayedReply = BackForwardItemAtIndexDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const std::optional<WebCore::BackForwardItemIdentifier>& itemID);
    using Reply = std::tuple<std::optional<WebCore::BackForwardItemIdentifier>&>;
    using ReplyArguments = std::tuple<std::optional<WebCore::BackForwardItemIdentifier>>;
    explicit BackForwardItemAtIndex(int32_t itemIndex)
        : m_arguments(itemIndex)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class BackForwardListCounts {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_BackForwardListCounts; }
    static constexpr bool isSync = true;

    using DelayedReply = BackForwardListCountsDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const WebKit::WebBackForwardListCounts& counts);
    using Reply = std::tuple<WebKit::WebBackForwardListCounts&>;
    using ReplyArguments = std::tuple<WebKit::WebBackForwardListCounts>;
    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class BackForwardClear {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_BackForwardClear; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class WillGoToBackForwardListItem {
public:
    using Arguments = std::tuple<const WebCore::BackForwardItemIdentifier&, bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_WillGoToBackForwardListItem; }
    static constexpr bool isSync = false;

    WillGoToBackForwardListItem(const WebCore::BackForwardItemIdentifier& itemID, bool inBackForwardCache)
        : m_arguments(itemID, inBackForwardCache)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class RegisterEditCommandForUndo {
public:
    using Arguments = std::tuple<uint64_t, const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RegisterEditCommandForUndo; }
    static constexpr bool isSync = false;

    RegisterEditCommandForUndo(uint64_t commandID, const String& label)
        : m_arguments(commandID, label)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class ClearAllEditCommands {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ClearAllEditCommands; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class RegisterInsertionUndoGrouping {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RegisterInsertionUndoGrouping; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class CanUndoRedo {
public:
    using Arguments = std::tuple<WebKit::UndoOrRedo>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_CanUndoRedo; }
    static constexpr bool isSync = true;

    using DelayedReply = CanUndoRedoDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, bool result);
    using Reply = std::tuple<bool&>;
    using ReplyArguments = std::tuple<bool>;
    explicit CanUndoRedo(WebKit::UndoOrRedo undoOrRedo)
        : m_arguments(undoOrRedo)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class ExecuteUndoRedo {
public:
    using Arguments = std::tuple<WebKit::UndoOrRedo>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ExecuteUndoRedo; }
    static constexpr bool isSync = true;

    using DelayedReply = ExecuteUndoRedoDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&);
    using Reply = std::tuple<>;
    using ReplyArguments = std::tuple<>;
    explicit ExecuteUndoRedo(WebKit::UndoOrRedo undoOrRedo)
        : m_arguments(undoOrRedo)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class LogDiagnosticMessageFromWebProcess {
public:
    using Arguments = std::tuple<const String&, const String&, WebCore::ShouldSample>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_LogDiagnosticMessageFromWebProcess; }
    static constexpr bool isSync = false;

    LogDiagnosticMessageFromWebProcess(const String& message, const String& description, WebCore::ShouldSample shouldSample)
        : m_arguments(message, description, shouldSample)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class LogDiagnosticMessageWithResultFromWebProcess {
public:
    using Arguments = std::tuple<const String&, const String&, uint32_t, WebCore::ShouldSample>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_LogDiagnosticMessageWithResultFromWebProcess; }
    static constexpr bool isSync = false;

    LogDiagnosticMessageWithResultFromWebProcess(const String& message, const String& description, uint32_t result, WebCore::ShouldSample shouldSample)
        : m_arguments(message, description, result, shouldSample)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class LogDiagnosticMessageWithValueFromWebProcess {
public:
    using Arguments = std::tuple<const String&, const String&, double, const unsigned&, WebCore::ShouldSample>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_LogDiagnosticMessageWithValueFromWebProcess; }
    static constexpr bool isSync = false;

    LogDiagnosticMessageWithValueFromWebProcess(const String& message, const String& description, double value, const unsigned& significantFigures, WebCore::ShouldSample shouldSample)
        : m_arguments(message, description, value, significantFigures, shouldSample)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class LogDiagnosticMessageWithEnhancedPrivacyFromWebProcess {
public:
    using Arguments = std::tuple<const String&, const String&, WebCore::ShouldSample>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_LogDiagnosticMessageWithEnhancedPrivacyFromWebProcess; }
    static constexpr bool isSync = false;

    LogDiagnosticMessageWithEnhancedPrivacyFromWebProcess(const String& message, const String& description, WebCore::ShouldSample shouldSample)
        : m_arguments(message, description, shouldSample)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class LogDiagnosticMessageWithValueDictionaryFromWebProcess {
public:
    using Arguments = std::tuple<const String&, const String&, const WebCore::DiagnosticLoggingClient::ValueDictionary&, WebCore::ShouldSample>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_LogDiagnosticMessageWithValueDictionaryFromWebProcess; }
    static constexpr bool isSync = false;

    LogDiagnosticMessageWithValueDictionaryFromWebProcess(const String& message, const String& description, const WebCore::DiagnosticLoggingClient::ValueDictionary& value, WebCore::ShouldSample shouldSample)
        : m_arguments(message, description, value, shouldSample)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class LogDiagnosticMessageWithDomainFromWebProcess {
public:
    using Arguments = std::tuple<const String&, const WebCore::DiagnosticLoggingDomain&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_LogDiagnosticMessageWithDomainFromWebProcess; }
    static constexpr bool isSync = false;

    LogDiagnosticMessageWithDomainFromWebProcess(const String& message, const WebCore::DiagnosticLoggingDomain& domain)
        : m_arguments(message, domain)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class LogScrollingEvent {
public:
    using Arguments = std::tuple<uint32_t, const MonotonicTime&, uint64_t>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_LogScrollingEvent; }
    static constexpr bool isSync = false;

    LogScrollingEvent(uint32_t eventType, const MonotonicTime& timestamp, uint64_t data)
        : m_arguments(eventType, timestamp, data)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class EditorStateChanged {
public:
    using Arguments = std::tuple<const WebKit::EditorState&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_EditorStateChanged; }
    static constexpr bool isSync = false;

    explicit EditorStateChanged(const WebKit::EditorState& editorState)
        : m_arguments(editorState)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class CompositionWasCanceled {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_CompositionWasCanceled; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SetHasHadSelectionChangesFromUserInteraction {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetHasHadSelectionChangesFromUserInteraction; }
    static constexpr bool isSync = false;

    explicit SetHasHadSelectionChangesFromUserInteraction(bool hasHadUserSelectionChanges)
        : m_arguments(hasHadUserSelectionChanges)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if ENABLE(IMAGE_ANALYSIS)
class RequestTextRecognition {
public:
    using Arguments = std::tuple<const URL&, const WebKit::ShareableBitmap::Handle&, const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RequestTextRecognition; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(WebCore::TextRecognitionResult&&)>&&);
    static void cancelReply(CompletionHandler<void(WebCore::TextRecognitionResult&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_RequestTextRecognitionReply; }
    using AsyncReply = RequestTextRecognitionAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const WebCore::TextRecognitionResult& result);
    using Reply = std::tuple<WebCore::TextRecognitionResult&>;
    using ReplyArguments = std::tuple<WebCore::TextRecognitionResult>;
    RequestTextRecognition(const URL& imageURL, const WebKit::ShareableBitmap::Handle& imageData, const String& identifier)
        : m_arguments(imageURL, imageData, identifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if HAVE(TRANSLATION_UI_SERVICES) && ENABLE(CONTEXT_MENUS)
class HandleContextMenuTranslation {
public:
    using Arguments = std::tuple<const WebCore::TranslationContextMenuInfo&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_HandleContextMenuTranslation; }
    static constexpr bool isSync = false;

    explicit HandleContextMenuTranslation(const WebCore::TranslationContextMenuInfo& info)
        : m_arguments(info)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(MEDIA_CONTROLS_CONTEXT_MENUS) && USE(UICONTEXTMENU)
class ShowMediaControlsContextMenu {
public:
    using Arguments = std::tuple<const WebCore::FloatRect&, const Vector<WebCore::MediaControlsContextMenuItem>&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ShowMediaControlsContextMenu; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(WebCore::MediaControlsContextMenuItem::ID&&)>&&);
    static void cancelReply(CompletionHandler<void(WebCore::MediaControlsContextMenuItem::ID&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_ShowMediaControlsContextMenuReply; }
    using AsyncReply = ShowMediaControlsContextMenuAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const WebCore::MediaControlsContextMenuItem::ID& selectedItemID);
    using Reply = std::tuple<WebCore::MediaControlsContextMenuItem::ID&>;
    using ReplyArguments = std::tuple<WebCore::MediaControlsContextMenuItem::ID>;
    ShowMediaControlsContextMenu(const WebCore::FloatRect& targetFrame, const Vector<WebCore::MediaControlsContextMenuItem>& items)
        : m_arguments(targetFrame, items)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if HAVE(TOUCH_BAR)
class SetIsTouchBarUpdateSupressedForHiddenContentEditable {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetIsTouchBarUpdateSupressedForHiddenContentEditable; }
    static constexpr bool isSync = false;

    explicit SetIsTouchBarUpdateSupressedForHiddenContentEditable(bool isTouchBarUpdateSupressed)
        : m_arguments(isTouchBarUpdateSupressed)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if HAVE(TOUCH_BAR)
class SetIsNeverRichlyEditableForTouchBar {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetIsNeverRichlyEditableForTouchBar; }
    static constexpr bool isSync = false;

    explicit SetIsNeverRichlyEditableForTouchBar(bool isNeverRichlyEditable)
        : m_arguments(isNeverRichlyEditable)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class RequestDOMPasteAccess {
public:
    using Arguments = std::tuple<WebCore::DOMPasteAccessCategory, const WebCore::IntRect&, const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RequestDOMPasteAccess; }
    static constexpr bool isSync = true;

    using DelayedReply = RequestDOMPasteAccessDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, WebCore::DOMPasteAccessResponse response);
    using Reply = std::tuple<WebCore::DOMPasteAccessResponse&>;
    using ReplyArguments = std::tuple<WebCore::DOMPasteAccessResponse>;
    RequestDOMPasteAccess(WebCore::DOMPasteAccessCategory pasteAccessCategory, const WebCore::IntRect& elementRect, const String& originIdentifier)
        : m_arguments(pasteAccessCategory, elementRect, originIdentifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidCountStringMatches {
public:
    using Arguments = std::tuple<const String&, uint32_t>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidCountStringMatches; }
    static constexpr bool isSync = false;

    DidCountStringMatches(const String& string, uint32_t matchCount)
        : m_arguments(string, matchCount)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SetTextIndicator {
public:
    using Arguments = std::tuple<const WebCore::TextIndicatorData&, uint64_t>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetTextIndicator; }
    static constexpr bool isSync = false;

    SetTextIndicator(const WebCore::TextIndicatorData& indicator, uint64_t lifetime)
        : m_arguments(indicator, lifetime)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class ClearTextIndicator {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ClearTextIndicator; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidFindString {
public:
    using Arguments = std::tuple<const String&, const Vector<WebCore::IntRect>&, uint32_t, int32_t, bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidFindString; }
    static constexpr bool isSync = false;

    DidFindString(const String& string, const Vector<WebCore::IntRect>& matchRect, uint32_t matchCount, int32_t matchIndex, bool didWrapAround)
        : m_arguments(string, matchRect, matchCount, matchIndex, didWrapAround)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidFailToFindString {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidFailToFindString; }
    static constexpr bool isSync = false;

    explicit DidFailToFindString(const String& string)
        : m_arguments(string)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidFindStringMatches {
public:
    using Arguments = std::tuple<const String&, const Vector<Vector<WebCore::IntRect>>&, int32_t>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidFindStringMatches; }
    static constexpr bool isSync = false;

    DidFindStringMatches(const String& string, const Vector<Vector<WebCore::IntRect>>& matches, int32_t firstIndexAfterSelection)
        : m_arguments(string, matches, firstIndexAfterSelection)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidGetImageForFindMatch {
public:
    using Arguments = std::tuple<const WebKit::ShareableBitmap::Handle&, uint32_t>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidGetImageForFindMatch; }
    static constexpr bool isSync = false;

    DidGetImageForFindMatch(const WebKit::ShareableBitmap::Handle& contentImageHandle, uint32_t matchIndex)
        : m_arguments(contentImageHandle, matchIndex)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class ShowPopupMenu {
public:
    using Arguments = std::tuple<const WebCore::IntRect&, uint64_t, const Vector<WebKit::WebPopupItem>&, int32_t, const WebKit::PlatformPopupMenuData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ShowPopupMenu; }
    static constexpr bool isSync = false;

    ShowPopupMenu(const WebCore::IntRect& rect, uint64_t textDirection, const Vector<WebKit::WebPopupItem>& items, int32_t selectedIndex, const WebKit::PlatformPopupMenuData& data)
        : m_arguments(rect, textDirection, items, selectedIndex, data)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class HidePopupMenu {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_HidePopupMenu; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if ENABLE(CONTEXT_MENUS)
class ShowContextMenu {
public:
    using Arguments = std::tuple<const WebKit::ContextMenuContextData&, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ShowContextMenu; }
    static constexpr bool isSync = false;

    ShowContextMenu(const WebKit::ContextMenuContextData& contextMenuContextData, const WebKit::UserData& userData)
        : m_arguments(contextMenuContextData, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(SERVICE_WORKER)
class DidFinishServiceWorkerPageRegistration {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidFinishServiceWorkerPageRegistration; }
    static constexpr bool isSync = false;

    explicit DidFinishServiceWorkerPageRegistration(bool success)
        : m_arguments(success)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class ExceededDatabaseQuota {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const String&, const String&, const String&, uint64_t, uint64_t, uint64_t, uint64_t>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ExceededDatabaseQuota; }
    static constexpr bool isSync = true;

    using DelayedReply = ExceededDatabaseQuotaDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, uint64_t newQuota);
    using Reply = std::tuple<uint64_t&>;
    using ReplyArguments = std::tuple<uint64_t>;
    ExceededDatabaseQuota(const WebCore::FrameIdentifier& frameID, const String& originIdentifier, const String& databaseName, const String& databaseDisplayName, uint64_t currentQuota, uint64_t currentOriginUsage, uint64_t currentDatabaseUsage, uint64_t expectedUsage)
        : m_arguments(frameID, originIdentifier, databaseName, databaseDisplayName, currentQuota, currentOriginUsage, currentDatabaseUsage, expectedUsage)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class ReachedApplicationCacheOriginQuota {
public:
    using Arguments = std::tuple<const String&, uint64_t, uint64_t>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ReachedApplicationCacheOriginQuota; }
    static constexpr bool isSync = true;

    using DelayedReply = ReachedApplicationCacheOriginQuotaDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, uint64_t newQuota);
    using Reply = std::tuple<uint64_t&>;
    using ReplyArguments = std::tuple<uint64_t>;
    ReachedApplicationCacheOriginQuota(const String& originIdentifier, uint64_t currentQuota, uint64_t totalBytesNeeded)
        : m_arguments(originIdentifier, currentQuota, totalBytesNeeded)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class RequestGeolocationPermissionForFrame {
public:
    using Arguments = std::tuple<const WebKit::GeolocationIdentifier&, const WebKit::FrameInfoData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RequestGeolocationPermissionForFrame; }
    static constexpr bool isSync = false;

    RequestGeolocationPermissionForFrame(const WebKit::GeolocationIdentifier& geolocationID, const WebKit::FrameInfoData& frameInfo)
        : m_arguments(geolocationID, frameInfo)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class RevokeGeolocationAuthorizationToken {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RevokeGeolocationAuthorizationToken; }
    static constexpr bool isSync = false;

    explicit RevokeGeolocationAuthorizationToken(const String& authorizationToken)
        : m_arguments(authorizationToken)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if ENABLE(MEDIA_STREAM)
class RequestUserMediaPermissionForFrame {
public:
    using Arguments = std::tuple<const WebCore::UserMediaRequestIdentifier&, const WebCore::FrameIdentifier&, const WebCore::SecurityOriginData&, const WebCore::SecurityOriginData&, const WebCore::MediaStreamRequest&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RequestUserMediaPermissionForFrame; }
    static constexpr bool isSync = false;

    RequestUserMediaPermissionForFrame(const WebCore::UserMediaRequestIdentifier& userMediaID, const WebCore::FrameIdentifier& frameID, const WebCore::SecurityOriginData& userMediaDocumentOriginIdentifier, const WebCore::SecurityOriginData& topLevelDocumentOriginIdentifier, const WebCore::MediaStreamRequest& request)
        : m_arguments(userMediaID, frameID, userMediaDocumentOriginIdentifier, topLevelDocumentOriginIdentifier, request)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(MEDIA_STREAM)
class EnumerateMediaDevicesForFrame {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebCore::SecurityOriginData&, const WebCore::SecurityOriginData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_EnumerateMediaDevicesForFrame; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(Vector<WebCore::CaptureDevice>&&, String&&)>&&);
    static void cancelReply(CompletionHandler<void(Vector<WebCore::CaptureDevice>&&, String&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_EnumerateMediaDevicesForFrameReply; }
    using AsyncReply = EnumerateMediaDevicesForFrameAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const Vector<WebCore::CaptureDevice>& devices, const String& mediaDeviceIdentifierHashSalt);
    using Reply = std::tuple<Vector<WebCore::CaptureDevice>&, String&>;
    using ReplyArguments = std::tuple<Vector<WebCore::CaptureDevice>, String>;
    EnumerateMediaDevicesForFrame(const WebCore::FrameIdentifier& frameID, const WebCore::SecurityOriginData& userMediaDocumentOriginIdentifier, const WebCore::SecurityOriginData& topLevelDocumentOriginIdentifier)
        : m_arguments(frameID, userMediaDocumentOriginIdentifier, topLevelDocumentOriginIdentifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(MEDIA_STREAM)
class BeginMonitoringCaptureDevices {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_BeginMonitoringCaptureDevices; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ENCRYPTED_MEDIA)
class RequestMediaKeySystemPermissionForFrame {
public:
    using Arguments = std::tuple<const WebCore::MediaKeySystemRequestIdentifier&, const WebCore::FrameIdentifier&, const WebCore::SecurityOriginData&, const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RequestMediaKeySystemPermissionForFrame; }
    static constexpr bool isSync = false;

    RequestMediaKeySystemPermissionForFrame(const WebCore::MediaKeySystemRequestIdentifier& mediaKeySystemID, const WebCore::FrameIdentifier& frameID, const WebCore::SecurityOriginData& topLevelDocumentOriginIdentifier, const String& keySystem)
        : m_arguments(mediaKeySystemID, frameID, topLevelDocumentOriginIdentifier, keySystem)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class RequestNotificationPermission {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RequestNotificationPermission; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(bool&&)>&&);
    static void cancelReply(CompletionHandler<void(bool&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_RequestNotificationPermissionReply; }
    using AsyncReply = RequestNotificationPermissionAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, bool allowed);
    using Reply = std::tuple<bool&>;
    using ReplyArguments = std::tuple<bool>;
    explicit RequestNotificationPermission(const String& originIdentifier)
        : m_arguments(originIdentifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if USE(UNIFIED_TEXT_CHECKING)
class CheckTextOfParagraph {
public:
    using Arguments = std::tuple<const String&, const OptionSet<WebCore::TextCheckingType>&, int32_t>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_CheckTextOfParagraph; }
    static constexpr bool isSync = true;

    using DelayedReply = CheckTextOfParagraphDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const Vector<WebCore::TextCheckingResult>& results);
    using Reply = std::tuple<Vector<WebCore::TextCheckingResult>&>;
    using ReplyArguments = std::tuple<Vector<WebCore::TextCheckingResult>>;
    CheckTextOfParagraph(const String& text, const OptionSet<WebCore::TextCheckingType>& checkingTypes, int32_t insertionPoint)
        : m_arguments(text, checkingTypes, insertionPoint)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class CheckSpellingOfString {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_CheckSpellingOfString; }
    static constexpr bool isSync = true;

    using DelayedReply = CheckSpellingOfStringDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, int32_t misspellingLocation, int32_t misspellingLength);
    using Reply = std::tuple<int32_t&, int32_t&>;
    using ReplyArguments = std::tuple<int32_t, int32_t>;
    explicit CheckSpellingOfString(const String& text)
        : m_arguments(text)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class CheckGrammarOfString {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_CheckGrammarOfString; }
    static constexpr bool isSync = true;

    using DelayedReply = CheckGrammarOfStringDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const Vector<WebCore::GrammarDetail>& results, int32_t badGrammarLocation, int32_t badGrammarLength);
    using Reply = std::tuple<Vector<WebCore::GrammarDetail>&, int32_t&, int32_t&>;
    using ReplyArguments = std::tuple<Vector<WebCore::GrammarDetail>, int32_t, int32_t>;
    explicit CheckGrammarOfString(const String& text)
        : m_arguments(text)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SpellingUIIsShowing {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SpellingUIIsShowing; }
    static constexpr bool isSync = true;

    using DelayedReply = SpellingUIIsShowingDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, bool isShowing);
    using Reply = std::tuple<bool&>;
    using ReplyArguments = std::tuple<bool>;
    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class UpdateSpellingUIWithMisspelledWord {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_UpdateSpellingUIWithMisspelledWord; }
    static constexpr bool isSync = false;

    explicit UpdateSpellingUIWithMisspelledWord(const String& misspelledWord)
        : m_arguments(misspelledWord)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class UpdateSpellingUIWithGrammarString {
public:
    using Arguments = std::tuple<const String&, const WebCore::GrammarDetail&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_UpdateSpellingUIWithGrammarString; }
    static constexpr bool isSync = false;

    UpdateSpellingUIWithGrammarString(const String& badGrammarPhrase, const WebCore::GrammarDetail& grammarDetail)
        : m_arguments(badGrammarPhrase, grammarDetail)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class GetGuessesForWord {
public:
    using Arguments = std::tuple<const String&, const String&, int32_t>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_GetGuessesForWord; }
    static constexpr bool isSync = true;

    using DelayedReply = GetGuessesForWordDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const Vector<String>& guesses);
    using Reply = std::tuple<Vector<String>&>;
    using ReplyArguments = std::tuple<Vector<String>>;
    GetGuessesForWord(const String& word, const String& context, int32_t insertionPoint)
        : m_arguments(word, context, insertionPoint)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class LearnWord {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_LearnWord; }
    static constexpr bool isSync = false;

    explicit LearnWord(const String& word)
        : m_arguments(word)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class IgnoreWord {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_IgnoreWord; }
    static constexpr bool isSync = false;

    explicit IgnoreWord(const String& word)
        : m_arguments(word)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class RequestCheckingOfString {
public:
    using Arguments = std::tuple<const WebKit::TextCheckerRequestID&, const WebCore::TextCheckingRequestData&, int32_t>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RequestCheckingOfString; }
    static constexpr bool isSync = false;

    RequestCheckingOfString(const WebKit::TextCheckerRequestID& requestID, const WebCore::TextCheckingRequestData& request, int32_t insertionPoint)
        : m_arguments(requestID, request, insertionPoint)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if ENABLE(DRAG_SUPPORT)
class DidPerformDragControllerAction {
public:
    using Arguments = std::tuple<const std::optional<WebCore::DragOperation>&, WebCore::DragHandlingMethod, bool, const unsigned&, const WebCore::IntRect&, const WebCore::IntRect&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidPerformDragControllerAction; }
    static constexpr bool isSync = false;

    DidPerformDragControllerAction(const std::optional<WebCore::DragOperation>& dragOperation, WebCore::DragHandlingMethod dragHandlingMethod, bool mouseIsOverFileInput, const unsigned& numberOfItemsToBeAccepted, const WebCore::IntRect& insertionRect, const WebCore::IntRect& editableElementRect)
        : m_arguments(dragOperation, dragHandlingMethod, mouseIsOverFileInput, numberOfItemsToBeAccepted, insertionRect, editableElementRect)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(DRAG_SUPPORT)
class DidEndDragging {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidEndDragging; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(COCOA) && ENABLE(DRAG_SUPPORT)
class StartDrag {
public:
    using Arguments = std::tuple<const WebCore::DragItem&, const WebKit::ShareableBitmap::Handle&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_StartDrag; }
    static constexpr bool isSync = false;

    StartDrag(const WebCore::DragItem& dragItem, const WebKit::ShareableBitmap::Handle& dragImage)
        : m_arguments(dragItem, dragImage)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(COCOA) && ENABLE(DRAG_SUPPORT)
class SetPromisedDataForImage {
public:
    using Arguments = std::tuple<const String&, const WebKit::SharedMemory::IPCHandle&, const String&, const String&, const String&, const String&, const String&, const WebKit::SharedMemory::IPCHandle&, const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetPromisedDataForImage; }
    static constexpr bool isSync = false;

    SetPromisedDataForImage(const String& pasteboardName, const WebKit::SharedMemory::IPCHandle& imageHandle, const String& filename, const String& extension, const String& title, const String& url, const String& visibleURL, const WebKit::SharedMemory::IPCHandle& archiveHandle, const String& originIdentifier)
        : m_arguments(pasteboardName, imageHandle, filename, extension, title, url, visibleURL, archiveHandle, originIdentifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(GTK) && ENABLE(DRAG_SUPPORT)
class StartDrag {
public:
    using Arguments = std::tuple<const WebCore::SelectionData&, const OptionSet<WebCore::DragOperation>&, const WebKit::ShareableBitmap::Handle&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_StartDrag; }
    static constexpr bool isSync = false;

    StartDrag(const WebCore::SelectionData& selectionData, const OptionSet<WebCore::DragOperation>& dragOperationMask, const WebKit::ShareableBitmap::Handle& dragImage)
        : m_arguments(selectionData, dragOperationMask, dragImage)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(DRAG_SUPPORT)
class DidPerformDragOperation {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidPerformDragOperation; }
    static constexpr bool isSync = false;

    explicit DidPerformDragOperation(bool handled)
        : m_arguments(handled)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY) && ENABLE(DRAG_SUPPORT)
class DidHandleDragStartRequest {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidHandleDragStartRequest; }
    static constexpr bool isSync = false;

    explicit DidHandleDragStartRequest(bool started)
        : m_arguments(started)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY) && ENABLE(DRAG_SUPPORT)
class DidHandleAdditionalDragItemsRequest {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidHandleAdditionalDragItemsRequest; }
    static constexpr bool isSync = false;

    explicit DidHandleAdditionalDragItemsRequest(bool added)
        : m_arguments(added)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY) && ENABLE(DRAG_SUPPORT)
class WillReceiveEditDragSnapshot {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_WillReceiveEditDragSnapshot; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY) && ENABLE(DRAG_SUPPORT)
class DidReceiveEditDragSnapshot {
public:
    using Arguments = std::tuple<const std::optional<WebCore::TextIndicatorData>&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidReceiveEditDragSnapshot; }
    static constexpr bool isSync = false;

    explicit DidReceiveEditDragSnapshot(const std::optional<WebCore::TextIndicatorData>& textIndicator)
        : m_arguments(textIndicator)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(COCOA)
class DidPerformDictionaryLookup {
public:
    using Arguments = std::tuple<const WebCore::DictionaryPopupInfo&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidPerformDictionaryLookup; }
    static constexpr bool isSync = false;

    explicit DidPerformDictionaryLookup(const WebCore::DictionaryPopupInfo& dictionaryPopupInfo)
        : m_arguments(dictionaryPopupInfo)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(COCOA)
class ExecuteSavedCommandBySelector {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ExecuteSavedCommandBySelector; }
    static constexpr bool isSync = true;

    using DelayedReply = ExecuteSavedCommandBySelectorDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, bool handled);
    using Reply = std::tuple<bool&>;
    using ReplyArguments = std::tuple<bool>;
    explicit ExecuteSavedCommandBySelector(const String& selector)
        : m_arguments(selector)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(COCOA)
class RegisterWebProcessAccessibilityToken {
public:
    using Arguments = std::tuple<const IPC::DataReference&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RegisterWebProcessAccessibilityToken; }
    static constexpr bool isSync = false;

    explicit RegisterWebProcessAccessibilityToken(const IPC::DataReference& data)
        : m_arguments(data)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(COCOA)
class GetIsSpeaking {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_GetIsSpeaking; }
    static constexpr bool isSync = true;

    using DelayedReply = GetIsSpeakingDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, bool isSpeaking);
    using Reply = std::tuple<bool&>;
    using ReplyArguments = std::tuple<bool>;
    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(COCOA)
class Speak {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_Speak; }
    static constexpr bool isSync = false;

    explicit Speak(const String& string)
        : m_arguments(string)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(COCOA)
class StopSpeaking {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_StopSpeaking; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(COCOA)
class MakeFirstResponder {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_MakeFirstResponder; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(COCOA)
class AssistiveTechnologyMakeFirstResponder {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_AssistiveTechnologyMakeFirstResponder; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(COCOA)
class SearchWithSpotlight {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SearchWithSpotlight; }
    static constexpr bool isSync = false;

    explicit SearchWithSpotlight(const String& string)
        : m_arguments(string)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(COCOA)
class SearchTheWeb {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SearchTheWeb; }
    static constexpr bool isSync = false;

    explicit SearchTheWeb(const String& string)
        : m_arguments(string)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if HAVE(TOUCH_BAR)
class TouchBarMenuDataChanged {
public:
    using Arguments = std::tuple<const WebKit::TouchBarMenuData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_TouchBarMenuDataChanged; }
    static constexpr bool isSync = false;

    explicit TouchBarMenuDataChanged(const WebKit::TouchBarMenuData& touchBarMenuData)
        : m_arguments(touchBarMenuData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if HAVE(TOUCH_BAR)
class TouchBarMenuItemDataAdded {
public:
    using Arguments = std::tuple<const WebKit::TouchBarMenuItemData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_TouchBarMenuItemDataAdded; }
    static constexpr bool isSync = false;

    explicit TouchBarMenuItemDataAdded(const WebKit::TouchBarMenuItemData& touchBarMenuItemData)
        : m_arguments(touchBarMenuItemData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if HAVE(TOUCH_BAR)
class TouchBarMenuItemDataRemoved {
public:
    using Arguments = std::tuple<const WebKit::TouchBarMenuItemData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_TouchBarMenuItemDataRemoved; }
    static constexpr bool isSync = false;

    explicit TouchBarMenuItemDataRemoved(const WebKit::TouchBarMenuItemData& touchBarMenuItemData)
        : m_arguments(touchBarMenuItemData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if USE(APPKIT)
class SubstitutionsPanelIsShowing {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SubstitutionsPanelIsShowing; }
    static constexpr bool isSync = true;

    using DelayedReply = SubstitutionsPanelIsShowingDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, bool isShowing);
    using Reply = std::tuple<bool&>;
    using ReplyArguments = std::tuple<bool>;
    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if USE(AUTOMATIC_TEXT_REPLACEMENT)
class toggleSmartInsertDelete {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_toggleSmartInsertDelete; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if USE(AUTOMATIC_TEXT_REPLACEMENT)
class toggleAutomaticQuoteSubstitution {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_toggleAutomaticQuoteSubstitution; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if USE(AUTOMATIC_TEXT_REPLACEMENT)
class toggleAutomaticLinkDetection {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_toggleAutomaticLinkDetection; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if USE(AUTOMATIC_TEXT_REPLACEMENT)
class toggleAutomaticDashSubstitution {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_toggleAutomaticDashSubstitution; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if USE(AUTOMATIC_TEXT_REPLACEMENT)
class toggleAutomaticTextReplacement {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_toggleAutomaticTextReplacement; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(MAC)
class ShowCorrectionPanel {
public:
    using Arguments = std::tuple<int32_t, const WebCore::FloatRect&, const String&, const String&, const Vector<String>&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ShowCorrectionPanel; }
    static constexpr bool isSync = false;

    ShowCorrectionPanel(int32_t panelType, const WebCore::FloatRect& boundingBoxOfReplacedString, const String& replacedString, const String& replacementString, const Vector<String>& alternativeReplacementStrings)
        : m_arguments(panelType, boundingBoxOfReplacedString, replacedString, replacementString, alternativeReplacementStrings)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(MAC)
class DismissCorrectionPanel {
public:
    using Arguments = std::tuple<int32_t>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DismissCorrectionPanel; }
    static constexpr bool isSync = false;

    explicit DismissCorrectionPanel(int32_t reason)
        : m_arguments(reason)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(MAC)
class DismissCorrectionPanelSoon {
public:
    using Arguments = std::tuple<int32_t>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DismissCorrectionPanelSoon; }
    static constexpr bool isSync = true;

    using DelayedReply = DismissCorrectionPanelSoonDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const String& result);
    using Reply = std::tuple<String&>;
    using ReplyArguments = std::tuple<String>;
    explicit DismissCorrectionPanelSoon(int32_t reason)
        : m_arguments(reason)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(MAC)
class RecordAutocorrectionResponse {
public:
    using Arguments = std::tuple<int32_t, const String&, const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RecordAutocorrectionResponse; }
    static constexpr bool isSync = false;

    RecordAutocorrectionResponse(int32_t response, const String& replacedString, const String& replacementString)
        : m_arguments(response, replacedString, replacementString)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(MAC)
class SetEditableElementIsFocused {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetEditableElementIsFocused; }
    static constexpr bool isSync = false;

    explicit SetEditableElementIsFocused(bool editableElementIsFocused)
        : m_arguments(editableElementIsFocused)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(MAC)
class HandleAcceptsFirstMouse {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_HandleAcceptsFirstMouse; }
    static constexpr bool isSync = false;

    explicit HandleAcceptsFirstMouse(bool acceptsFirstMouse)
        : m_arguments(acceptsFirstMouse)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class DidUpdateRenderingAfterCommittingLoad {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidUpdateRenderingAfterCommittingLoad; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if USE(DICTATION_ALTERNATIVES)
class ShowDictationAlternativeUI {
public:
    using Arguments = std::tuple<const WebCore::FloatRect&, const WebCore::DictationContext&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ShowDictationAlternativeUI; }
    static constexpr bool isSync = false;

    ShowDictationAlternativeUI(const WebCore::FloatRect& boundingBoxOfDictatedText, const WebCore::DictationContext& dictationContext)
        : m_arguments(boundingBoxOfDictatedText, dictationContext)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if USE(DICTATION_ALTERNATIVES)
class RemoveDictationAlternatives {
public:
    using Arguments = std::tuple<const WebCore::DictationContext&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RemoveDictationAlternatives; }
    static constexpr bool isSync = false;

    explicit RemoveDictationAlternatives(const WebCore::DictationContext& dictationContext)
        : m_arguments(dictationContext)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if USE(DICTATION_ALTERNATIVES)
class DictationAlternatives {
public:
    using Arguments = std::tuple<const WebCore::DictationContext&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DictationAlternatives; }
    static constexpr bool isSync = true;

    using DelayedReply = DictationAlternativesDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const Vector<String>& alternatives);
    using Reply = std::tuple<Vector<String>&>;
    using ReplyArguments = std::tuple<Vector<String>>;
    explicit DictationAlternatives(const WebCore::DictationContext& dictationContext)
        : m_arguments(dictationContext)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class CouldNotRestorePageState {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_CouldNotRestorePageState; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class RestorePageState {
public:
    using Arguments = std::tuple<const std::optional<WebCore::FloatPoint>&, const WebCore::FloatPoint&, const WebCore::RectEdges<float>&, double>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RestorePageState; }
    static constexpr bool isSync = false;

    RestorePageState(const std::optional<WebCore::FloatPoint>& scrollPosition, const WebCore::FloatPoint& scrollOrigin, const WebCore::RectEdges<float>& obscuredInsetsOnSave, double scale)
        : m_arguments(scrollPosition, scrollOrigin, obscuredInsetsOnSave, scale)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class RestorePageCenterAndScale {
public:
    using Arguments = std::tuple<const std::optional<WebCore::FloatPoint>&, double>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RestorePageCenterAndScale; }
    static constexpr bool isSync = false;

    RestorePageCenterAndScale(const std::optional<WebCore::FloatPoint>& unobscuredCenter, double scale)
        : m_arguments(unobscuredCenter, scale)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class DidGetTapHighlightGeometries {
public:
    using Arguments = std::tuple<const WebKit::TapIdentifier&, const WebCore::Color&, const Vector<WebCore::FloatQuad>&, const WebCore::IntSize&, const WebCore::IntSize&, const WebCore::IntSize&, const WebCore::IntSize&, bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidGetTapHighlightGeometries; }
    static constexpr bool isSync = false;

    DidGetTapHighlightGeometries(const WebKit::TapIdentifier& requestID, const WebCore::Color& color, const Vector<WebCore::FloatQuad>& geometries, const WebCore::IntSize& topLeftRadius, const WebCore::IntSize& topRightRadius, const WebCore::IntSize& bottomLeftRadius, const WebCore::IntSize& bottomRightRadius, bool nodeHasBuiltInClickHandling)
        : m_arguments(requestID, color, geometries, topLeftRadius, topRightRadius, bottomLeftRadius, bottomRightRadius, nodeHasBuiltInClickHandling)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class ElementDidFocus {
public:
    using Arguments = std::tuple<const WebKit::FocusedElementInformation&, bool, bool, const OptionSet<WebCore::ActivityState::Flag>&, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ElementDidFocus; }
    static constexpr bool isSync = false;

    ElementDidFocus(const WebKit::FocusedElementInformation& information, bool userIsInteracting, bool blurPreviousNode, const OptionSet<WebCore::ActivityState::Flag>& activityStateChanges, const WebKit::UserData& userData)
        : m_arguments(information, userIsInteracting, blurPreviousNode, activityStateChanges, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class ElementDidBlur {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ElementDidBlur; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class UpdateInputContextAfterBlurringAndRefocusingElement {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_UpdateInputContextAfterBlurringAndRefocusingElement; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class FocusedElementDidChangeInputMode {
public:
    using Arguments = std::tuple<WebCore::InputMode>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_FocusedElementDidChangeInputMode; }
    static constexpr bool isSync = false;

    explicit FocusedElementDidChangeInputMode(WebCore::InputMode mode)
        : m_arguments(mode)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class ScrollingNodeScrollWillStartScroll {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ScrollingNodeScrollWillStartScroll; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class ScrollingNodeScrollDidEndScroll {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ScrollingNodeScrollDidEndScroll; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class ShowInspectorHighlight {
public:
    using Arguments = std::tuple<const WebCore::InspectorOverlay::Highlight&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ShowInspectorHighlight; }
    static constexpr bool isSync = false;

    explicit ShowInspectorHighlight(const WebCore::InspectorOverlay::Highlight& highlight)
        : m_arguments(highlight)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class HideInspectorHighlight {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_HideInspectorHighlight; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class ShowInspectorIndication {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ShowInspectorIndication; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class HideInspectorIndication {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_HideInspectorIndication; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class EnableInspectorNodeSearch {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_EnableInspectorNodeSearch; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class DisableInspectorNodeSearch {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DisableInspectorNodeSearch; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class UpdateStringForFind {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_UpdateStringForFind; }
    static constexpr bool isSync = false;

    explicit UpdateStringForFind(const String& findString)
        : m_arguments(findString)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class HandleAutocorrectionContext {
public:
    using Arguments = std::tuple<const WebKit::WebAutocorrectionContext&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_HandleAutocorrectionContext; }
    static constexpr bool isSync = false;

    explicit HandleAutocorrectionContext(const WebKit::WebAutocorrectionContext& context)
        : m_arguments(context)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(IOS_FAMILY)
class ShowDataDetectorsUIForPositionInformation {
public:
    using Arguments = std::tuple<const WebKit::InteractionInformationAtPosition&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ShowDataDetectorsUIForPositionInformation; }
    static constexpr bool isSync = false;

    explicit ShowDataDetectorsUIForPositionInformation(const WebKit::InteractionInformationAtPosition& information)
        : m_arguments(information)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class DidChangeInspectorFrontendCount {
public:
    using Arguments = std::tuple<uint64_t>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidChangeInspectorFrontendCount; }
    static constexpr bool isSync = false;

    explicit DidChangeInspectorFrontendCount(uint64_t count)
        : m_arguments(count)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class CreateInspectorTarget {
public:
    using Arguments = std::tuple<const String&, Inspector::InspectorTargetType>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_CreateInspectorTarget; }
    static constexpr bool isSync = false;

    CreateInspectorTarget(const String& targetId, Inspector::InspectorTargetType type)
        : m_arguments(targetId, type)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DestroyInspectorTarget {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DestroyInspectorTarget; }
    static constexpr bool isSync = false;

    explicit DestroyInspectorTarget(const String& targetId)
        : m_arguments(targetId)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SendMessageToInspectorFrontend {
public:
    using Arguments = std::tuple<const String&, const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SendMessageToInspectorFrontend; }
    static constexpr bool isSync = false;

    SendMessageToInspectorFrontend(const String& targetId, const String& message)
        : m_arguments(targetId, message)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class SaveRecentSearches {
public:
    using Arguments = std::tuple<const String&, const Vector<WebCore::RecentSearch>&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SaveRecentSearches; }
    static constexpr bool isSync = false;

    SaveRecentSearches(const String& name, const Vector<WebCore::RecentSearch>& searchItems)
        : m_arguments(name, searchItems)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class LoadRecentSearches {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_LoadRecentSearches; }
    static constexpr bool isSync = true;

    using DelayedReply = LoadRecentSearchesDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const Vector<WebCore::RecentSearch>& result);
    using Reply = std::tuple<Vector<WebCore::RecentSearch>&>;
    using ReplyArguments = std::tuple<Vector<WebCore::RecentSearch>>;
    explicit LoadRecentSearches(const String& name)
        : m_arguments(name)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if ENABLE(PDFKIT_PLUGIN) && !ENABLE(UI_PROCESS_PDF_HUD)
class SavePDFToFileInDownloadsFolder {
public:
    using Arguments = std::tuple<const String&, const URL&, const IPC::DataReference&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SavePDFToFileInDownloadsFolder; }
    static constexpr bool isSync = false;

    SavePDFToFileInDownloadsFolder(const String& suggestedFilename, const URL& originatingURL, const IPC::DataReference& data)
        : m_arguments(suggestedFilename, originatingURL, data)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(PDFKIT_PLUGIN) && !ENABLE(UI_PROCESS_PDF_HUD)
class SavePDFToTemporaryFolderAndOpenWithNativeApplication {
public:
    using Arguments = std::tuple<const String&, const WebKit::FrameInfoData&, const IPC::DataReference&, const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SavePDFToTemporaryFolderAndOpenWithNativeApplication; }
    static constexpr bool isSync = false;

    SavePDFToTemporaryFolderAndOpenWithNativeApplication(const String& suggestedFilename, const WebKit::FrameInfoData& sourceFrameInfo, const IPC::DataReference& data, const String& pdfUUID)
        : m_arguments(suggestedFilename, sourceFrameInfo, data, pdfUUID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(PDFKIT_PLUGIN) && !ENABLE(UI_PROCESS_PDF_HUD)
class OpenPDFFromTemporaryFolderWithNativeApplication {
public:
    using Arguments = std::tuple<const WebKit::FrameInfoData&, const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_OpenPDFFromTemporaryFolderWithNativeApplication; }
    static constexpr bool isSync = false;

    OpenPDFFromTemporaryFolderWithNativeApplication(const WebKit::FrameInfoData& sourceFrameInfo, const String& pdfUUID)
        : m_arguments(sourceFrameInfo, pdfUUID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(PDFKIT_PLUGIN)
class ShowPDFContextMenu {
public:
    using Arguments = std::tuple<const WebKit::PDFContextMenu&, const WebKit::PDFPluginIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ShowPDFContextMenu; }
    static constexpr bool isSync = true;

    using DelayedReply = ShowPDFContextMenuDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const std::optional<int32_t>& selectedItem);
    using Reply = std::tuple<std::optional<int32_t>&>;
    using ReplyArguments = std::tuple<std::optional<int32_t>>;
    ShowPDFContextMenu(const WebKit::PDFContextMenu& contextMenu, const WebKit::PDFPluginIdentifier& identifier)
        : m_arguments(contextMenu, identifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class DidUpdateActivityState {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidUpdateActivityState; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if ENABLE(WEB_CRYPTO)
class WrapCryptoKey {
public:
    using Arguments = std::tuple<const Vector<uint8_t>&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_WrapCryptoKey; }
    static constexpr bool isSync = true;

    using DelayedReply = WrapCryptoKeyDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, bool succeeded, const Vector<uint8_t>& wrappedKey);
    using Reply = std::tuple<bool&, Vector<uint8_t>&>;
    using ReplyArguments = std::tuple<bool, Vector<uint8_t>>;
    explicit WrapCryptoKey(const Vector<uint8_t>& key)
        : m_arguments(key)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(WEB_CRYPTO)
class UnwrapCryptoKey {
public:
    using Arguments = std::tuple<const Vector<uint8_t>&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_UnwrapCryptoKey; }
    static constexpr bool isSync = true;

    using DelayedReply = UnwrapCryptoKeyDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, bool succeeded, const Vector<uint8_t>& key);
    using Reply = std::tuple<bool&, Vector<uint8_t>&>;
    using ReplyArguments = std::tuple<bool, Vector<uint8_t>>;
    explicit UnwrapCryptoKey(const Vector<uint8_t>& wrappedKey)
        : m_arguments(wrappedKey)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if (ENABLE(TELEPHONE_NUMBER_DETECTION) && PLATFORM(MAC))
class ShowTelephoneNumberMenu {
public:
    using Arguments = std::tuple<const String&, const WebCore::IntPoint&, const WebCore::IntRect&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ShowTelephoneNumberMenu; }
    static constexpr bool isSync = false;

    ShowTelephoneNumberMenu(const String& telephoneNumber, const WebCore::IntPoint& point, const WebCore::IntRect& rect)
        : m_arguments(telephoneNumber, point, rect)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if USE(QUICK_LOOK)
class DidStartLoadForQuickLookDocumentInMainFrame {
public:
    using Arguments = std::tuple<const String&, const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidStartLoadForQuickLookDocumentInMainFrame; }
    static constexpr bool isSync = false;

    DidStartLoadForQuickLookDocumentInMainFrame(const String& fileName, const String& uti)
        : m_arguments(fileName, uti)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if USE(QUICK_LOOK)
class DidFinishLoadForQuickLookDocumentInMainFrame {
public:
    using Arguments = std::tuple<const WebKit::ShareableResource::Handle&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidFinishLoadForQuickLookDocumentInMainFrame; }
    static constexpr bool isSync = false;

    explicit DidFinishLoadForQuickLookDocumentInMainFrame(const WebKit::ShareableResource::Handle& resource)
        : m_arguments(resource)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if USE(QUICK_LOOK)
class RequestPasswordForQuickLookDocumentInMainFrame {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RequestPasswordForQuickLookDocumentInMainFrame; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(String&&)>&&);
    static void cancelReply(CompletionHandler<void(String&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_RequestPasswordForQuickLookDocumentInMainFrameReply; }
    using AsyncReply = RequestPasswordForQuickLookDocumentInMainFrameAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const String& password);
    using Reply = std::tuple<String&>;
    using ReplyArguments = std::tuple<String>;
    explicit RequestPasswordForQuickLookDocumentInMainFrame(const String& fileName)
        : m_arguments(fileName)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(CONTENT_FILTERING)
class ContentFilterDidBlockLoadForFrame {
public:
    using Arguments = std::tuple<const WebCore::ContentFilterUnblockHandler&, const WebCore::FrameIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ContentFilterDidBlockLoadForFrame; }
    static constexpr bool isSync = false;

    ContentFilterDidBlockLoadForFrame(const WebCore::ContentFilterUnblockHandler& unblockHandler, const WebCore::FrameIdentifier& frameID)
        : m_arguments(unblockHandler, frameID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class IsPlayingMediaDidChange {
public:
    using Arguments = std::tuple<const OptionSet<WebCore::MediaProducerMediaState>&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_IsPlayingMediaDidChange; }
    static constexpr bool isSync = false;

    explicit IsPlayingMediaDidChange(const OptionSet<WebCore::MediaProducerMediaState>& state)
        : m_arguments(state)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class HandleAutoplayEvent {
public:
    using Arguments = std::tuple<WebCore::AutoplayEvent, const OptionSet<WebCore::AutoplayEventFlags>&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_HandleAutoplayEvent; }
    static constexpr bool isSync = false;

    HandleAutoplayEvent(WebCore::AutoplayEvent event, const OptionSet<WebCore::AutoplayEventFlags>& flags)
        : m_arguments(event, flags)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if PLATFORM(MAC)
class DidPerformImmediateActionHitTest {
public:
    using Arguments = std::tuple<const WebKit::WebHitTestResultData&, bool, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidPerformImmediateActionHitTest; }
    static constexpr bool isSync = false;

    DidPerformImmediateActionHitTest(const WebKit::WebHitTestResultData& result, bool contentPreventsDefault, const WebKit::UserData& userData)
        : m_arguments(result, contentPreventsDefault, userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class HandleMessage {
public:
    using Arguments = std::tuple<const String&, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_HandleMessage; }
    static constexpr bool isSync = false;

    HandleMessage(const String& messageName, const WebKit::UserData& messageBody)
        : m_arguments(messageName, messageBody)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class HandleSynchronousMessage {
public:
    using Arguments = std::tuple<const String&, const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_HandleSynchronousMessage; }
    static constexpr bool isSync = true;

    using DelayedReply = HandleSynchronousMessageDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const WebKit::UserData& returnData);
    using Reply = std::tuple<WebKit::UserData&>;
    using ReplyArguments = std::tuple<WebKit::UserData>;
    HandleSynchronousMessage(const String& messageName, const WebKit::UserData& messageBody)
        : m_arguments(messageName, messageBody)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class HandleAutoFillButtonClick {
public:
    using Arguments = std::tuple<const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_HandleAutoFillButtonClick; }
    static constexpr bool isSync = false;

    explicit HandleAutoFillButtonClick(const WebKit::UserData& userData)
        : m_arguments(userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidResignInputElementStrongPasswordAppearance {
public:
    using Arguments = std::tuple<const WebKit::UserData&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidResignInputElementStrongPasswordAppearance; }
    static constexpr bool isSync = false;

    explicit DidResignInputElementStrongPasswordAppearance(const WebKit::UserData& userData)
        : m_arguments(userData)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if ENABLE(CONTENT_EXTENSIONS)
class ContentRuleListNotification {
public:
    using Arguments = std::tuple<const URL&, const WebCore::ContentRuleListResults&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ContentRuleListNotification; }
    static constexpr bool isSync = false;

    ContentRuleListNotification(const URL& url, const WebCore::ContentRuleListResults& results)
        : m_arguments(url, results)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(WIRELESS_PLAYBACK_TARGET) && !PLATFORM(IOS_FAMILY)
class AddPlaybackTargetPickerClient {
public:
    using Arguments = std::tuple<const WebCore::PlaybackTargetClientContextIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_AddPlaybackTargetPickerClient; }
    static constexpr bool isSync = false;

    explicit AddPlaybackTargetPickerClient(const WebCore::PlaybackTargetClientContextIdentifier& contextId)
        : m_arguments(contextId)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(WIRELESS_PLAYBACK_TARGET) && !PLATFORM(IOS_FAMILY)
class RemovePlaybackTargetPickerClient {
public:
    using Arguments = std::tuple<const WebCore::PlaybackTargetClientContextIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RemovePlaybackTargetPickerClient; }
    static constexpr bool isSync = false;

    explicit RemovePlaybackTargetPickerClient(const WebCore::PlaybackTargetClientContextIdentifier& contextId)
        : m_arguments(contextId)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(WIRELESS_PLAYBACK_TARGET) && !PLATFORM(IOS_FAMILY)
class ShowPlaybackTargetPicker {
public:
    using Arguments = std::tuple<const WebCore::PlaybackTargetClientContextIdentifier&, const WebCore::FloatRect&, bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ShowPlaybackTargetPicker; }
    static constexpr bool isSync = false;

    ShowPlaybackTargetPicker(const WebCore::PlaybackTargetClientContextIdentifier& contextId, const WebCore::FloatRect& pickerLocation, bool hasVideo)
        : m_arguments(contextId, pickerLocation, hasVideo)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(WIRELESS_PLAYBACK_TARGET) && !PLATFORM(IOS_FAMILY)
class PlaybackTargetPickerClientStateDidChange {
public:
    using Arguments = std::tuple<const WebCore::PlaybackTargetClientContextIdentifier&, const OptionSet<WebCore::MediaProducerMediaState>&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_PlaybackTargetPickerClientStateDidChange; }
    static constexpr bool isSync = false;

    PlaybackTargetPickerClientStateDidChange(const WebCore::PlaybackTargetClientContextIdentifier& contextId, const OptionSet<WebCore::MediaProducerMediaState>& mediaState)
        : m_arguments(contextId, mediaState)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(WIRELESS_PLAYBACK_TARGET) && !PLATFORM(IOS_FAMILY)
class SetMockMediaPlaybackTargetPickerEnabled {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetMockMediaPlaybackTargetPickerEnabled; }
    static constexpr bool isSync = false;

    explicit SetMockMediaPlaybackTargetPickerEnabled(bool enabled)
        : m_arguments(enabled)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(WIRELESS_PLAYBACK_TARGET) && !PLATFORM(IOS_FAMILY)
class SetMockMediaPlaybackTargetPickerState {
public:
    using Arguments = std::tuple<const String&, WebCore::MediaPlaybackTargetContext::MockState>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetMockMediaPlaybackTargetPickerState; }
    static constexpr bool isSync = false;

    SetMockMediaPlaybackTargetPickerState(const String& name, WebCore::MediaPlaybackTargetContext::MockState pickerState)
        : m_arguments(name, pickerState)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(WIRELESS_PLAYBACK_TARGET) && !PLATFORM(IOS_FAMILY)
class MockMediaPlaybackTargetPickerDismissPopup {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_MockMediaPlaybackTargetPickerDismissPopup; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(VIDEO_PRESENTATION_MODE)
class SetMockVideoPresentationModeEnabled {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetMockVideoPresentationModeEnabled; }
    static constexpr bool isSync = false;

    explicit SetMockVideoPresentationModeEnabled(bool enabled)
        : m_arguments(enabled)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(POINTER_LOCK)
class RequestPointerLock {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RequestPointerLock; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(POINTER_LOCK)
class RequestPointerUnlock {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RequestPointerUnlock; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class DidFailToSuspendAfterProcessSwap {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidFailToSuspendAfterProcessSwap; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DidSuspendAfterProcessSwap {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidSuspendAfterProcessSwap; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class ImageOrMediaDocumentSizeChanged {
public:
    using Arguments = std::tuple<const WebCore::IntSize&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ImageOrMediaDocumentSizeChanged; }
    static constexpr bool isSync = false;

    explicit ImageOrMediaDocumentSizeChanged(const WebCore::IntSize& newSize)
        : m_arguments(newSize)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class UseFixedLayoutDidChange {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_UseFixedLayoutDidChange; }
    static constexpr bool isSync = false;

    explicit UseFixedLayoutDidChange(bool useFixedLayout)
        : m_arguments(useFixedLayout)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class FixedLayoutSizeDidChange {
public:
    using Arguments = std::tuple<const WebCore::IntSize&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_FixedLayoutSizeDidChange; }
    static constexpr bool isSync = false;

    explicit FixedLayoutSizeDidChange(const WebCore::IntSize& fixedLayoutSize)
        : m_arguments(fixedLayoutSize)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if ENABLE(VIDEO) && USE(GSTREAMER)
class RequestInstallMissingMediaPlugins {
public:
    using Arguments = std::tuple<const String&, const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RequestInstallMissingMediaPlugins; }
    static constexpr bool isSync = false;

    RequestInstallMissingMediaPlugins(const String& details, const String& description)
        : m_arguments(details, description)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class DidRestoreScrollPosition {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidRestoreScrollPosition; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class RequestScrollToRect {
public:
    using Arguments = std::tuple<const WebCore::FloatRect&, const WebCore::FloatPoint&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RequestScrollToRect; }
    static constexpr bool isSync = false;

    RequestScrollToRect(const WebCore::FloatRect& targetRect, const WebCore::FloatPoint& origin)
        : m_arguments(targetRect, origin)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class GetLoadDecisionForIcon {
public:
    using Arguments = std::tuple<const WebCore::LinkIcon&, const WebKit::CallbackID&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_GetLoadDecisionForIcon; }
    static constexpr bool isSync = false;

    GetLoadDecisionForIcon(const WebCore::LinkIcon& icon, const WebKit::CallbackID& callbackID)
        : m_arguments(icon, callbackID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if PLATFORM(MAC)
class DidHandleAcceptedCandidate {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidHandleAcceptedCandidate; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class SetIsUsingHighPerformanceWebGL {
public:
    using Arguments = std::tuple<bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetIsUsingHighPerformanceWebGL; }
    static constexpr bool isSync = false;

    explicit SetIsUsingHighPerformanceWebGL(bool isUsingHighPerformanceWebGL)
        : m_arguments(isUsingHighPerformanceWebGL)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class StartURLSchemeTask {
public:
    using Arguments = std::tuple<const WebKit::URLSchemeTaskParameters&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_StartURLSchemeTask; }
    static constexpr bool isSync = false;

    explicit StartURLSchemeTask(const WebKit::URLSchemeTaskParameters& parameters)
        : m_arguments(parameters)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class StopURLSchemeTask {
public:
    using Arguments = std::tuple<const WebKit::WebURLSchemeHandlerIdentifier&, const WebCore::ResourceLoaderIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_StopURLSchemeTask; }
    static constexpr bool isSync = false;

    StopURLSchemeTask(const WebKit::WebURLSchemeHandlerIdentifier& handlerIdentifier, const WebCore::ResourceLoaderIdentifier& taskIdentifier)
        : m_arguments(handlerIdentifier, taskIdentifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class LoadSynchronousURLSchemeTask {
public:
    using Arguments = std::tuple<const WebKit::URLSchemeTaskParameters&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_LoadSynchronousURLSchemeTask; }
    static constexpr bool isSync = true;

    using DelayedReply = LoadSynchronousURLSchemeTaskDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const WebCore::ResourceResponse& response, const WebCore::ResourceError& error, const Vector<uint8_t>& data);
    using Reply = std::tuple<WebCore::ResourceResponse&, WebCore::ResourceError&, Vector<uint8_t>&>;
    using ReplyArguments = std::tuple<WebCore::ResourceResponse, WebCore::ResourceError, Vector<uint8_t>>;
    explicit LoadSynchronousURLSchemeTask(const WebKit::URLSchemeTaskParameters& parameters)
        : m_arguments(parameters)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if ENABLE(DEVICE_ORIENTATION)
class ShouldAllowDeviceOrientationAndMotionAccess {
public:
    using Arguments = std::tuple<const WebCore::FrameIdentifier&, const WebKit::FrameInfoData&, bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ShouldAllowDeviceOrientationAndMotionAccess; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(WebCore::DeviceOrientationOrMotionPermissionState&&)>&&);
    static void cancelReply(CompletionHandler<void(WebCore::DeviceOrientationOrMotionPermissionState&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_ShouldAllowDeviceOrientationAndMotionAccessReply; }
    using AsyncReply = ShouldAllowDeviceOrientationAndMotionAccessAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, WebCore::DeviceOrientationOrMotionPermissionState permissionState);
    using Reply = std::tuple<WebCore::DeviceOrientationOrMotionPermissionState&>;
    using ReplyArguments = std::tuple<WebCore::DeviceOrientationOrMotionPermissionState>;
    ShouldAllowDeviceOrientationAndMotionAccess(const WebCore::FrameIdentifier& frameID, const WebKit::FrameInfoData& frameInfo, bool mayPrompt)
        : m_arguments(frameID, frameInfo, mayPrompt)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ATTACHMENT_ELEMENT)
class RegisterAttachmentIdentifierFromData {
public:
    using Arguments = std::tuple<const String&, const String&, const String&, const IPC::SharedBufferCopy&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RegisterAttachmentIdentifierFromData; }
    static constexpr bool isSync = false;

    RegisterAttachmentIdentifierFromData(const String& identifier, const String& contentType, const String& preferredFileName, const IPC::SharedBufferCopy& data)
        : m_arguments(identifier, contentType, preferredFileName, data)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ATTACHMENT_ELEMENT)
class RegisterAttachmentIdentifierFromFilePath {
public:
    using Arguments = std::tuple<const String&, const String&, const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RegisterAttachmentIdentifierFromFilePath; }
    static constexpr bool isSync = false;

    RegisterAttachmentIdentifierFromFilePath(const String& identifier, const String& contentType, const String& filePath)
        : m_arguments(identifier, contentType, filePath)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ATTACHMENT_ELEMENT)
class RegisterAttachmentIdentifier {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RegisterAttachmentIdentifier; }
    static constexpr bool isSync = false;

    explicit RegisterAttachmentIdentifier(const String& identifier)
        : m_arguments(identifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ATTACHMENT_ELEMENT)
class RegisterAttachmentsFromSerializedData {
public:
    using Arguments = std::tuple<const Vector<WebCore::SerializedAttachmentData>&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RegisterAttachmentsFromSerializedData; }
    static constexpr bool isSync = false;

    explicit RegisterAttachmentsFromSerializedData(const Vector<WebCore::SerializedAttachmentData>& data)
        : m_arguments(data)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ATTACHMENT_ELEMENT)
class CloneAttachmentData {
public:
    using Arguments = std::tuple<const String&, const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_CloneAttachmentData; }
    static constexpr bool isSync = false;

    CloneAttachmentData(const String& fromIdentifier, const String& toIdentifier)
        : m_arguments(fromIdentifier, toIdentifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ATTACHMENT_ELEMENT)
class DidInsertAttachmentWithIdentifier {
public:
    using Arguments = std::tuple<const String&, const String&, bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidInsertAttachmentWithIdentifier; }
    static constexpr bool isSync = false;

    DidInsertAttachmentWithIdentifier(const String& identifier, const String& source, bool hasEnclosingImage)
        : m_arguments(identifier, source, hasEnclosingImage)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ATTACHMENT_ELEMENT)
class DidRemoveAttachmentWithIdentifier {
public:
    using Arguments = std::tuple<const String&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidRemoveAttachmentWithIdentifier; }
    static constexpr bool isSync = false;

    explicit DidRemoveAttachmentWithIdentifier(const String& identifier)
        : m_arguments(identifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ATTACHMENT_ELEMENT)
class SerializedAttachmentDataForIdentifiers {
public:
    using Arguments = std::tuple<const Vector<String>&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SerializedAttachmentDataForIdentifiers; }
    static constexpr bool isSync = true;

    using DelayedReply = SerializedAttachmentDataForIdentifiersDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const Vector<WebCore::SerializedAttachmentData>& seralizedData);
    using Reply = std::tuple<Vector<WebCore::SerializedAttachmentData>&>;
    using ReplyArguments = std::tuple<Vector<WebCore::SerializedAttachmentData>>;
    explicit SerializedAttachmentDataForIdentifiers(const Vector<String>& identifiers)
        : m_arguments(identifiers)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ATTACHMENT_ELEMENT)
class WritePromisedAttachmentToPasteboard {
public:
    using Arguments = std::tuple<const WebCore::PromisedAttachmentInfo&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_WritePromisedAttachmentToPasteboard; }
    static constexpr bool isSync = false;

    explicit WritePromisedAttachmentToPasteboard(const WebCore::PromisedAttachmentInfo& info)
        : m_arguments(info)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ATTACHMENT_ELEMENT)
class RequestAttachmentIcon {
public:
    using Arguments = std::tuple<const String&, const String&, const String&, const String&, const WebCore::FloatSize&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RequestAttachmentIcon; }
    static constexpr bool isSync = false;

    RequestAttachmentIcon(const String& identifier, const String& contentType, const String& path, const String& title, const WebCore::FloatSize& size)
        : m_arguments(identifier, contentType, path, title, size)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(APP_HIGHLIGHTS)
class StoreAppHighlight {
public:
    using Arguments = std::tuple<const WebCore::AppHighlight&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_StoreAppHighlight; }
    static constexpr bool isSync = false;

    explicit StoreAppHighlight(const WebCore::AppHighlight& info)
        : m_arguments(info)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class SignedPublicKeyAndChallengeString {
public:
    using Arguments = std::tuple<const unsigned&, const String&, const URL&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SignedPublicKeyAndChallengeString; }
    static constexpr bool isSync = true;

    using DelayedReply = SignedPublicKeyAndChallengeStringDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const String& result);
    using Reply = std::tuple<String&>;
    using ReplyArguments = std::tuple<String>;
    SignedPublicKeyAndChallengeString(const unsigned& keySizeIndex, const String& challengeString, const URL& url)
        : m_arguments(keySizeIndex, challengeString, url)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if ENABLE(SPEECH_SYNTHESIS)
class SpeechSynthesisVoiceList {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SpeechSynthesisVoiceList; }
    static constexpr bool isSync = true;

    using DelayedReply = SpeechSynthesisVoiceListDelayedReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const Vector<WebKit::WebSpeechSynthesisVoice>& voiceList);
    using Reply = std::tuple<Vector<WebKit::WebSpeechSynthesisVoice>&>;
    using ReplyArguments = std::tuple<Vector<WebKit::WebSpeechSynthesisVoice>>;
    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(SPEECH_SYNTHESIS)
class SpeechSynthesisSpeak {
public:
    using Arguments = std::tuple<const String&, const String&, float, float, float, const MonotonicTime&, const String&, const String&, const String&, bool, bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SpeechSynthesisSpeak; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void()>&&);
    static void cancelReply(CompletionHandler<void()>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_SpeechSynthesisSpeakReply; }
    using AsyncReply = SpeechSynthesisSpeakAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&);
    using Reply = std::tuple<>;
    using ReplyArguments = std::tuple<>;
    SpeechSynthesisSpeak(const String& text, const String& lang, float volume, float rate, float pitch, const MonotonicTime& startTime, const String& voiceURI, const String& voiceName, const String& voiceLang, bool localService, bool defaultVoice)
        : m_arguments(text, lang, volume, rate, pitch, startTime, voiceURI, voiceName, voiceLang, localService, defaultVoice)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(SPEECH_SYNTHESIS)
class SpeechSynthesisSetFinishedCallback {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SpeechSynthesisSetFinishedCallback; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void()>&&);
    static void cancelReply(CompletionHandler<void()>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_SpeechSynthesisSetFinishedCallbackReply; }
    using AsyncReply = SpeechSynthesisSetFinishedCallbackAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&);
    using Reply = std::tuple<>;
    using ReplyArguments = std::tuple<>;
    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(SPEECH_SYNTHESIS)
class SpeechSynthesisCancel {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SpeechSynthesisCancel; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(SPEECH_SYNTHESIS)
class SpeechSynthesisPause {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SpeechSynthesisPause; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void()>&&);
    static void cancelReply(CompletionHandler<void()>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_SpeechSynthesisPauseReply; }
    using AsyncReply = SpeechSynthesisPauseAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&);
    using Reply = std::tuple<>;
    using ReplyArguments = std::tuple<>;
    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(SPEECH_SYNTHESIS)
class SpeechSynthesisResume {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SpeechSynthesisResume; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void()>&&);
    static void cancelReply(CompletionHandler<void()>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_SpeechSynthesisResumeReply; }
    using AsyncReply = SpeechSynthesisResumeAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&);
    using Reply = std::tuple<>;
    using ReplyArguments = std::tuple<>;
    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(SPEECH_SYNTHESIS)
class SpeechSynthesisResetState {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SpeechSynthesisResetState; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(UI_PROCESS_PDF_HUD)
class CreatePDFHUD {
public:
    using Arguments = std::tuple<const WebKit::PDFPluginIdentifier&, const WebCore::IntRect&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_CreatePDFHUD; }
    static constexpr bool isSync = false;

    CreatePDFHUD(const WebKit::PDFPluginIdentifier& identifier, const WebCore::IntRect& boundingBox)
        : m_arguments(identifier, boundingBox)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(UI_PROCESS_PDF_HUD)
class UpdatePDFHUDLocation {
public:
    using Arguments = std::tuple<const WebKit::PDFPluginIdentifier&, const WebCore::IntRect&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_UpdatePDFHUDLocation; }
    static constexpr bool isSync = false;

    UpdatePDFHUDLocation(const WebKit::PDFPluginIdentifier& identifier, const WebCore::IntRect& boundingBox)
        : m_arguments(identifier, boundingBox)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(UI_PROCESS_PDF_HUD)
class RemovePDFHUD {
public:
    using Arguments = std::tuple<const WebKit::PDFPluginIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RemovePDFHUD; }
    static constexpr bool isSync = false;

    explicit RemovePDFHUD(const WebKit::PDFPluginIdentifier& identifier)
        : m_arguments(identifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class ConfigureLoggingChannel {
public:
    using Arguments = std::tuple<const String&, WTFLogChannelState, WTFLogLevel>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ConfigureLoggingChannel; }
    static constexpr bool isSync = false;

    ConfigureLoggingChannel(const String& channelName, WTFLogChannelState state, WTFLogLevel level)
        : m_arguments(channelName, state, level)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if PLATFORM(GTK)
class ShowEmojiPicker {
public:
    using Arguments = std::tuple<const WebCore::IntRect&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ShowEmojiPicker; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(String&&)>&&);
    static void cancelReply(CompletionHandler<void(String&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_ShowEmojiPickerReply; }
    using AsyncReply = ShowEmojiPickerAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const String& result);
    using Reply = std::tuple<String&>;
    using ReplyArguments = std::tuple<String>;
    explicit ShowEmojiPicker(const WebCore::IntRect& caretRect)
        : m_arguments(caretRect)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if HAVE(VISIBILITY_PROPAGATION_VIEW)
class DidCreateContextInWebProcessForVisibilityPropagation {
public:
    using Arguments = std::tuple<const WebKit::LayerHostingContextID&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidCreateContextInWebProcessForVisibilityPropagation; }
    static constexpr bool isSync = false;

    explicit DidCreateContextInWebProcessForVisibilityPropagation(const WebKit::LayerHostingContextID& contextID)
        : m_arguments(contextID)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(WEB_AUTHN)
class SetMockWebAuthenticationConfiguration {
public:
    using Arguments = std::tuple<const WebCore::MockWebAuthenticationConfiguration&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetMockWebAuthenticationConfiguration; }
    static constexpr bool isSync = false;

    explicit SetMockWebAuthenticationConfiguration(const WebCore::MockWebAuthenticationConfiguration& configuration)
        : m_arguments(configuration)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(GTK) || PLATFORM(WPE)
class SendMessageToWebView {
public:
    using Arguments = std::tuple<const WebKit::UserMessage&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SendMessageToWebView; }
    static constexpr bool isSync = false;

    explicit SendMessageToWebView(const WebKit::UserMessage& userMessage)
        : m_arguments(userMessage)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if PLATFORM(GTK) || PLATFORM(WPE)
class SendMessageToWebViewWithReply {
public:
    using Arguments = std::tuple<const WebKit::UserMessage&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SendMessageToWebViewWithReply; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(WebKit::UserMessage&&)>&&);
    static void cancelReply(CompletionHandler<void(WebKit::UserMessage&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_SendMessageToWebViewWithReplyReply; }
    using AsyncReply = SendMessageToWebViewWithReplyAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const WebKit::UserMessage& replyMessage);
    using Reply = std::tuple<WebKit::UserMessage&>;
    using ReplyArguments = std::tuple<WebKit::UserMessage>;
    explicit SendMessageToWebViewWithReply(const WebKit::UserMessage& userMessage)
        : m_arguments(userMessage)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class DidFindTextManipulationItems {
public:
    using Arguments = std::tuple<const Vector<WebCore::TextManipulationController::ManipulationItem>&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DidFindTextManipulationItems; }
    static constexpr bool isSync = false;

    explicit DidFindTextManipulationItems(const Vector<WebCore::TextManipulationController::ManipulationItem>& items)
        : m_arguments(items)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if ENABLE(MEDIA_USAGE)
class AddMediaUsageManagerSession {
public:
    using Arguments = std::tuple<const WebCore::MediaSessionIdentifier&, const String&, const URL&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_AddMediaUsageManagerSession; }
    static constexpr bool isSync = false;

    AddMediaUsageManagerSession(const WebCore::MediaSessionIdentifier& identifier, const String& bundleIdentifier, const URL& pageURL)
        : m_arguments(identifier, bundleIdentifier, pageURL)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(MEDIA_USAGE)
class UpdateMediaUsageManagerSessionState {
public:
    using Arguments = std::tuple<const WebCore::MediaSessionIdentifier&, const WebCore::MediaUsageInfo&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_UpdateMediaUsageManagerSessionState; }
    static constexpr bool isSync = false;

    UpdateMediaUsageManagerSessionState(const WebCore::MediaSessionIdentifier& identifier, const WebCore::MediaUsageInfo& info)
        : m_arguments(identifier, info)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(MEDIA_USAGE)
class RemoveMediaUsageManagerSession {
public:
    using Arguments = std::tuple<const WebCore::MediaSessionIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RemoveMediaUsageManagerSession; }
    static constexpr bool isSync = false;

    explicit RemoveMediaUsageManagerSession(const WebCore::MediaSessionIdentifier& identifier)
        : m_arguments(identifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class SetHasExecutedAppBoundBehaviorBeforeNavigation {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_SetHasExecutedAppBoundBehaviorBeforeNavigation; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if PLATFORM(MAC)
class ChangeUniversalAccessZoomFocus {
public:
    using Arguments = std::tuple<const WebCore::IntRect&, const WebCore::IntRect&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ChangeUniversalAccessZoomFocus; }
    static constexpr bool isSync = false;

    ChangeUniversalAccessZoomFocus(const WebCore::IntRect& viewRect, const WebCore::IntRect& caretRect)
        : m_arguments(viewRect, caretRect)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ARKIT_INLINE_PREVIEW_IOS)
class TakeModelElementFullscreen {
public:
    using Arguments = std::tuple<const WebKit::ModelIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_TakeModelElementFullscreen; }
    static constexpr bool isSync = false;

    explicit TakeModelElementFullscreen(const WebKit::ModelIdentifier& modelIdentifier)
        : m_arguments(modelIdentifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ARKIT_INLINE_PREVIEW_IOS)
class ModelElementSetInteractionEnabled {
public:
    using Arguments = std::tuple<const WebKit::ModelIdentifier&, bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ModelElementSetInteractionEnabled; }
    static constexpr bool isSync = false;

    ModelElementSetInteractionEnabled(const WebKit::ModelIdentifier& modelIdentifier, bool isInteractionEnabled)
        : m_arguments(modelIdentifier, isInteractionEnabled)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ARKIT_INLINE_PREVIEW_MAC)
class ModelElementDidCreatePreview {
public:
    using Arguments = std::tuple<const URL&, const String&, const WebCore::FloatSize&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ModelElementDidCreatePreview; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(Expected<std::pair<String, uint32_t>, WebCore::ResourceError>&&)>&&);
    static void cancelReply(CompletionHandler<void(Expected<std::pair<String, uint32_t>, WebCore::ResourceError>&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_ModelElementDidCreatePreviewReply; }
    using AsyncReply = ModelElementDidCreatePreviewAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const Expected<std::pair<String, uint32_t>, WebCore::ResourceError>& result);
    using Reply = std::tuple<Expected<std::pair<String, uint32_t>, WebCore::ResourceError>&>;
    using ReplyArguments = std::tuple<Expected<std::pair<String, uint32_t>, WebCore::ResourceError>>;
    ModelElementDidCreatePreview(const URL& url, const String& uuid, const WebCore::FloatSize& size)
        : m_arguments(url, uuid, size)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ARKIT_INLINE_PREVIEW_MAC)
class ModelElementSizeDidChange {
public:
    using Arguments = std::tuple<const String&, const WebCore::FloatSize&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ModelElementSizeDidChange; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(Expected<MachSendRight, WebCore::ResourceError>&&)>&&);
    static void cancelReply(CompletionHandler<void(Expected<MachSendRight, WebCore::ResourceError>&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_ModelElementSizeDidChangeReply; }
    using AsyncReply = ModelElementSizeDidChangeAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const Expected<MachSendRight, WebCore::ResourceError>& result);
    using Reply = std::tuple<Expected<MachSendRight, WebCore::ResourceError>&>;
    using ReplyArguments = std::tuple<Expected<MachSendRight, WebCore::ResourceError>>;
    ModelElementSizeDidChange(const String& uuid, const WebCore::FloatSize& size)
        : m_arguments(uuid, size)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ARKIT_INLINE_PREVIEW_MAC)
class HandleMouseDownForModelElement {
public:
    using Arguments = std::tuple<const String&, const WebCore::LayoutPoint&, const MonotonicTime&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_HandleMouseDownForModelElement; }
    static constexpr bool isSync = false;

    HandleMouseDownForModelElement(const String& uuid, const WebCore::LayoutPoint& flippedLocationInElement, const MonotonicTime& timestamp)
        : m_arguments(uuid, flippedLocationInElement, timestamp)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ARKIT_INLINE_PREVIEW_MAC)
class HandleMouseMoveForModelElement {
public:
    using Arguments = std::tuple<const String&, const WebCore::LayoutPoint&, const MonotonicTime&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_HandleMouseMoveForModelElement; }
    static constexpr bool isSync = false;

    HandleMouseMoveForModelElement(const String& uuid, const WebCore::LayoutPoint& flippedLocationInElement, const MonotonicTime& timestamp)
        : m_arguments(uuid, flippedLocationInElement, timestamp)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ARKIT_INLINE_PREVIEW_MAC)
class HandleMouseUpForModelElement {
public:
    using Arguments = std::tuple<const String&, const WebCore::LayoutPoint&, const MonotonicTime&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_HandleMouseUpForModelElement; }
    static constexpr bool isSync = false;

    HandleMouseUpForModelElement(const String& uuid, const WebCore::LayoutPoint& flippedLocationInElement, const MonotonicTime& timestamp)
        : m_arguments(uuid, flippedLocationInElement, timestamp)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ARKIT_INLINE_PREVIEW)
class ModelElementGetCamera {
public:
    using Arguments = std::tuple<const WebKit::ModelIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ModelElementGetCamera; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(Expected<WebCore::HTMLModelElementCamera, WebCore::ResourceError>&&)>&&);
    static void cancelReply(CompletionHandler<void(Expected<WebCore::HTMLModelElementCamera, WebCore::ResourceError>&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_ModelElementGetCameraReply; }
    using AsyncReply = ModelElementGetCameraAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const Expected<WebCore::HTMLModelElementCamera, WebCore::ResourceError>& result);
    using Reply = std::tuple<Expected<WebCore::HTMLModelElementCamera, WebCore::ResourceError>&>;
    using ReplyArguments = std::tuple<Expected<WebCore::HTMLModelElementCamera, WebCore::ResourceError>>;
    explicit ModelElementGetCamera(const WebKit::ModelIdentifier& modelIdentifier)
        : m_arguments(modelIdentifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ARKIT_INLINE_PREVIEW)
class ModelElementSetCamera {
public:
    using Arguments = std::tuple<const WebKit::ModelIdentifier&, const WebCore::HTMLModelElementCamera&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ModelElementSetCamera; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(bool&&)>&&);
    static void cancelReply(CompletionHandler<void(bool&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_ModelElementSetCameraReply; }
    using AsyncReply = ModelElementSetCameraAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, bool success);
    using Reply = std::tuple<bool&>;
    using ReplyArguments = std::tuple<bool>;
    ModelElementSetCamera(const WebKit::ModelIdentifier& modelIdentifier, const WebCore::HTMLModelElementCamera& camera)
        : m_arguments(modelIdentifier, camera)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ARKIT_INLINE_PREVIEW)
class ModelElementIsPlayingAnimation {
public:
    using Arguments = std::tuple<const WebKit::ModelIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ModelElementIsPlayingAnimation; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(Expected<bool, WebCore::ResourceError>&&)>&&);
    static void cancelReply(CompletionHandler<void(Expected<bool, WebCore::ResourceError>&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_ModelElementIsPlayingAnimationReply; }
    using AsyncReply = ModelElementIsPlayingAnimationAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const Expected<bool, WebCore::ResourceError>& result);
    using Reply = std::tuple<Expected<bool, WebCore::ResourceError>&>;
    using ReplyArguments = std::tuple<Expected<bool, WebCore::ResourceError>>;
    explicit ModelElementIsPlayingAnimation(const WebKit::ModelIdentifier& modelIdentifier)
        : m_arguments(modelIdentifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ARKIT_INLINE_PREVIEW)
class ModelElementSetAnimationIsPlaying {
public:
    using Arguments = std::tuple<const WebKit::ModelIdentifier&, bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ModelElementSetAnimationIsPlaying; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(bool&&)>&&);
    static void cancelReply(CompletionHandler<void(bool&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_ModelElementSetAnimationIsPlayingReply; }
    using AsyncReply = ModelElementSetAnimationIsPlayingAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, bool success);
    using Reply = std::tuple<bool&>;
    using ReplyArguments = std::tuple<bool>;
    ModelElementSetAnimationIsPlaying(const WebKit::ModelIdentifier& modelIdentifier, bool isPlaying)
        : m_arguments(modelIdentifier, isPlaying)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ARKIT_INLINE_PREVIEW)
class ModelElementIsLoopingAnimation {
public:
    using Arguments = std::tuple<const WebKit::ModelIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ModelElementIsLoopingAnimation; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(Expected<bool, WebCore::ResourceError>&&)>&&);
    static void cancelReply(CompletionHandler<void(Expected<bool, WebCore::ResourceError>&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_ModelElementIsLoopingAnimationReply; }
    using AsyncReply = ModelElementIsLoopingAnimationAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const Expected<bool, WebCore::ResourceError>& result);
    using Reply = std::tuple<Expected<bool, WebCore::ResourceError>&>;
    using ReplyArguments = std::tuple<Expected<bool, WebCore::ResourceError>>;
    explicit ModelElementIsLoopingAnimation(const WebKit::ModelIdentifier& modelIdentifier)
        : m_arguments(modelIdentifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ARKIT_INLINE_PREVIEW)
class ModelElementSetIsLoopingAnimation {
public:
    using Arguments = std::tuple<const WebKit::ModelIdentifier&, bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ModelElementSetIsLoopingAnimation; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(bool&&)>&&);
    static void cancelReply(CompletionHandler<void(bool&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_ModelElementSetIsLoopingAnimationReply; }
    using AsyncReply = ModelElementSetIsLoopingAnimationAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, bool success);
    using Reply = std::tuple<bool&>;
    using ReplyArguments = std::tuple<bool>;
    ModelElementSetIsLoopingAnimation(const WebKit::ModelIdentifier& modelIdentifier, bool isLooping)
        : m_arguments(modelIdentifier, isLooping)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ARKIT_INLINE_PREVIEW)
class ModelElementAnimationDuration {
public:
    using Arguments = std::tuple<const WebKit::ModelIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ModelElementAnimationDuration; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(Expected<Seconds, WebCore::ResourceError>&&)>&&);
    static void cancelReply(CompletionHandler<void(Expected<Seconds, WebCore::ResourceError>&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_ModelElementAnimationDurationReply; }
    using AsyncReply = ModelElementAnimationDurationAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const Expected<Seconds, WebCore::ResourceError>& result);
    using Reply = std::tuple<Expected<Seconds, WebCore::ResourceError>&>;
    using ReplyArguments = std::tuple<Expected<Seconds, WebCore::ResourceError>>;
    explicit ModelElementAnimationDuration(const WebKit::ModelIdentifier& modelIdentifier)
        : m_arguments(modelIdentifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ARKIT_INLINE_PREVIEW)
class ModelElementAnimationCurrentTime {
public:
    using Arguments = std::tuple<const WebKit::ModelIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ModelElementAnimationCurrentTime; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(Expected<Seconds, WebCore::ResourceError>&&)>&&);
    static void cancelReply(CompletionHandler<void(Expected<Seconds, WebCore::ResourceError>&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_ModelElementAnimationCurrentTimeReply; }
    using AsyncReply = ModelElementAnimationCurrentTimeAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const Expected<Seconds, WebCore::ResourceError>& result);
    using Reply = std::tuple<Expected<Seconds, WebCore::ResourceError>&>;
    using ReplyArguments = std::tuple<Expected<Seconds, WebCore::ResourceError>>;
    explicit ModelElementAnimationCurrentTime(const WebKit::ModelIdentifier& modelIdentifier)
        : m_arguments(modelIdentifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ARKIT_INLINE_PREVIEW)
class ModelElementSetAnimationCurrentTime {
public:
    using Arguments = std::tuple<const WebKit::ModelIdentifier&, const Seconds&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ModelElementSetAnimationCurrentTime; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(bool&&)>&&);
    static void cancelReply(CompletionHandler<void(bool&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_ModelElementSetAnimationCurrentTimeReply; }
    using AsyncReply = ModelElementSetAnimationCurrentTimeAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, bool success);
    using Reply = std::tuple<bool&>;
    using ReplyArguments = std::tuple<bool>;
    ModelElementSetAnimationCurrentTime(const WebKit::ModelIdentifier& modelIdentifier, const Seconds& currentTime)
        : m_arguments(modelIdentifier, currentTime)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ARKIT_INLINE_PREVIEW)
class ModelElementHasAudio {
public:
    using Arguments = std::tuple<const WebKit::ModelIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ModelElementHasAudio; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(Expected<bool, WebCore::ResourceError>&&)>&&);
    static void cancelReply(CompletionHandler<void(Expected<bool, WebCore::ResourceError>&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_ModelElementHasAudioReply; }
    using AsyncReply = ModelElementHasAudioAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const Expected<bool, WebCore::ResourceError>& result);
    using Reply = std::tuple<Expected<bool, WebCore::ResourceError>&>;
    using ReplyArguments = std::tuple<Expected<bool, WebCore::ResourceError>>;
    explicit ModelElementHasAudio(const WebKit::ModelIdentifier& modelIdentifier)
        : m_arguments(modelIdentifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ARKIT_INLINE_PREVIEW)
class ModelElementIsMuted {
public:
    using Arguments = std::tuple<const WebKit::ModelIdentifier&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ModelElementIsMuted; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(Expected<bool, WebCore::ResourceError>&&)>&&);
    static void cancelReply(CompletionHandler<void(Expected<bool, WebCore::ResourceError>&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_ModelElementIsMutedReply; }
    using AsyncReply = ModelElementIsMutedAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const Expected<bool, WebCore::ResourceError>& result);
    using Reply = std::tuple<Expected<bool, WebCore::ResourceError>&>;
    using ReplyArguments = std::tuple<Expected<bool, WebCore::ResourceError>>;
    explicit ModelElementIsMuted(const WebKit::ModelIdentifier& modelIdentifier)
        : m_arguments(modelIdentifier)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(ARKIT_INLINE_PREVIEW)
class ModelElementSetIsMuted {
public:
    using Arguments = std::tuple<const WebKit::ModelIdentifier&, bool>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ModelElementSetIsMuted; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(bool&&)>&&);
    static void cancelReply(CompletionHandler<void(bool&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_ModelElementSetIsMutedReply; }
    using AsyncReply = ModelElementSetIsMutedAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, bool success);
    using Reply = std::tuple<bool&>;
    using ReplyArguments = std::tuple<bool>;
    ModelElementSetIsMuted(const WebKit::ModelIdentifier& modelIdentifier, bool isMuted)
        : m_arguments(modelIdentifier, isMuted)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class QueryPermission {
public:
    using Arguments = std::tuple<const WebCore::ClientOrigin&, const WebCore::PermissionDescriptor&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_QueryPermission; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(std::optional<WebCore::PermissionState>&&, bool&&)>&&);
    static void cancelReply(CompletionHandler<void(std::optional<WebCore::PermissionState>&&, bool&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_QueryPermissionReply; }
    using AsyncReply = QueryPermissionAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const std::optional<WebCore::PermissionState>& state, bool shouldCache);
    using Reply = std::tuple<std::optional<WebCore::PermissionState>&, bool&>;
    using ReplyArguments = std::tuple<std::optional<WebCore::PermissionState>, bool>;
    QueryPermission(const WebCore::ClientOrigin& origin, const WebCore::PermissionDescriptor& descriptor)
        : m_arguments(origin, descriptor)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

#if ENABLE(APPLE_PAY_AMS_UI)
class StartApplePayAMSUISession {
public:
    using Arguments = std::tuple<const URL&, const WebCore::ApplePayAMSUIRequest&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_StartApplePayAMSUISession; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(std::optional<bool>&&)>&&);
    static void cancelReply(CompletionHandler<void(std::optional<bool>&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_StartApplePayAMSUISessionReply; }
    using AsyncReply = StartApplePayAMSUISessionAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const std::optional<bool>& result);
    using Reply = std::tuple<std::optional<bool>&>;
    using ReplyArguments = std::tuple<std::optional<bool>>;
    StartApplePayAMSUISession(const URL& originatingURL, const WebCore::ApplePayAMSUIRequest& request)
        : m_arguments(originatingURL, request)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

#if ENABLE(APPLE_PAY_AMS_UI)
class AbortApplePayAMSUISession {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_AbortApplePayAMSUISession; }
    static constexpr bool isSync = false;

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};
#endif

class RequestCookieConsent {
public:
    using Arguments = std::tuple<>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_RequestCookieConsent; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(WebCore::CookieConsentDecisionResult&&)>&&);
    static void cancelReply(CompletionHandler<void(WebCore::CookieConsentDecisionResult&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_RequestCookieConsentReply; }
    using AsyncReply = RequestCookieConsentAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, WebCore::CookieConsentDecisionResult result);
    using Reply = std::tuple<WebCore::CookieConsentDecisionResult&>;
    using ReplyArguments = std::tuple<WebCore::CookieConsentDecisionResult>;
    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class ClassifyModalContainerControls {
public:
    using Arguments = std::tuple<const Vector<String>&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_ClassifyModalContainerControls; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(Vector<WebCore::ModalContainerControlType>&&)>&&);
    static void cancelReply(CompletionHandler<void(Vector<WebCore::ModalContainerControlType>&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_ClassifyModalContainerControlsReply; }
    using AsyncReply = ClassifyModalContainerControlsAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, const Vector<WebCore::ModalContainerControlType>& types);
    using Reply = std::tuple<Vector<WebCore::ModalContainerControlType>&>;
    using ReplyArguments = std::tuple<Vector<WebCore::ModalContainerControlType>>;
    explicit ClassifyModalContainerControls(const Vector<String>& texts)
        : m_arguments(texts)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

class DecidePolicyForModalContainer {
public:
    using Arguments = std::tuple<const OptionSet<WebCore::ModalContainerControlType>&>;

    static IPC::MessageName name() { return IPC::MessageName::WebPageProxy_DecidePolicyForModalContainer; }
    static constexpr bool isSync = false;

    static void callReply(IPC::Decoder&, CompletionHandler<void(WebCore::ModalContainerDecision&&)>&&);
    static void cancelReply(CompletionHandler<void(WebCore::ModalContainerDecision&&)>&&);
    static IPC::MessageName asyncMessageReplyName() { return IPC::MessageName::WebPageProxy_DecidePolicyForModalContainerReply; }
    using AsyncReply = DecidePolicyForModalContainerAsyncReply;
    static constexpr auto callbackThread = WTF::CompletionHandlerCallThread::ConstructionThread;
    static void send(UniqueRef<IPC::Encoder>&&, IPC::Connection&, WebCore::ModalContainerDecision decision);
    using Reply = std::tuple<WebCore::ModalContainerDecision&>;
    using ReplyArguments = std::tuple<WebCore::ModalContainerDecision>;
    explicit DecidePolicyForModalContainer(const OptionSet<WebCore::ModalContainerControlType>& types)
        : m_arguments(types)
    {
    }

    const Arguments& arguments() const
    {
        return m_arguments;
    }

private:
    Arguments m_arguments;
};

} // namespace WebPageProxy
} // namespace Messages
